''' Unit tests for the HHblits wrapper
'''

import unittest
import os
import sys
import tempfile
import shutil
import filecmp
import datetime

import ost
from ost import seq
from ost import settings
from ost.bindings import hhblits3

class _UnitTestHHblitsLog(ost.LogSink):
  """Dedicated logger to hide some expected warning/ error messages.
  """
  def __init__(self):
    ost.LogSink.__init__(self)
    self.lcwd = os.getcwd()

  def LogMessage(self, message, severity):
    message = message.strip()
    dnem = "could not open file '%s'" % os.path.join(self.lcwd,
                                                     'doesnotexist.a3m')
    if message.endswith(dnem):
        return
    print(message)

def setUpModule():
  ost.PushLogSink(_UnitTestHHblitsLog())

class TestHHblitsBindings(unittest.TestCase):
    def setUp(self):
        # expects default hhblits installation...
        hhblits_bin_dir = os.path.dirname(settings.Locate('hhblits'))
        self.hhroot = os.path.join(hhblits_bin_dir, os.pardir)

    def tearDown(self):
        if hasattr(self, 'hh'):
            self.hh.Cleanup()
        if hasattr(self, 'tmpfile'):
            os.remove(self.tmpfile)

    def testOSTSequenceNoWDir(self):
        # OST sequence as input, no working dir should work
        query_seq = seq.CreateSequence('1AKE.B',
                                       'MRIILLGAPGAGKGTQAQFIMEKYGIPQISTGDMLRA'+
                                       'AVKSGSELGKQAKDIMDAGKLVTDELVIALVKERIAQ'+
                                       'EDCRNGFLLDGFPRTIPQADAMKEAGINVDYVLEFDV'+
                                       'PDELIVDRIVGRRVHAPSGRVYHVKFNPPKVEGKDDV'+
                                       'TGEELTTRKDDQEETVRKRLVEYHQMTAPLIGYYYYS'+
                                       'KEAEAGNTKYAKVDGTKPVAEVRADLEKILG')
        self.hh = hhblits3.HHblits(query_seq, self.hhroot)
        # this works only as long as utils.TempDirWithFiles() names the first
        # fasta file seq01.fasta
        self.assertEqual(self.hh.filename[-11:], 'seq01.fasta')

    def testOSTSequenceWDir(self):
        # OST sequence as input, with working dir should work
        query_seq = seq.CreateSequence('1AKE.B',
                                       'MRIILLGAPGAGKGTQAQFIMEKYGIPQISTGDMLRA'+
                                       'AVKSGSELGKQAKDIMDAGKLVTDELVIALVKERIAQ'+
                                       'EDCRNGFLLDGFPRTIPQADAMKEAGINVDYVLEFDV'+
                                       'PDELIVDRIVGRRVHAPSGRVYHVKFNPPKVEGKDDV'+
                                       'TGEELTTRKDDQEETVRKRLVEYHQMTAPLIGYYYYS'+
                                       'KEAEAGNTKYAKVDGTKPVAEVRADLEKILG')
        tmpdir = tempfile.mkdtemp()
        self.hh = hhblits3.HHblits(query_seq, self.hhroot, working_dir=tmpdir)
        self.assertEqual(self.hh.filename, os.path.join(tmpdir,
                                                        'query_hhblits.fasta'))
        self.assertEqual(self.hh.working_dir, tmpdir)
        self.hh.needs_cleanup = True

    def testFileNoWDir(self):
        # we have a sequence but no working dir
        self.hh = hhblits3.HHblits('testfiles/seq2.fasta', self.hhroot)
        self.assertEqual(self.hh.filename[-10:], 'seq2.fasta')

    def testFileWDir(self):
        # we have a sequence and a working dir
        tmpdir = tempfile.mkdtemp()
        self.hh = hhblits3.HHblits('testfiles/seq2.fasta', self.hhroot,
                                   working_dir=tmpdir)
        self.assertEqual(self.hh.filename, os.path.join(tmpdir, 'seq2.fasta'))
        self.assertEqual(self.hh.working_dir, tmpdir)
        self.hh.needs_cleanup = True

    def testDoCleanup(self):
        # test that cleanup cleans up
        tmpdir = tempfile.mkdtemp()
        self.hh = hhblits3.HHblits('testfiles/seq2.fasta', self.hhroot,
                                   working_dir=tmpdir)
        self.assertTrue(os.path.isdir(tmpdir))
        self.hh.needs_cleanup = True
        self.hh.Cleanup()
        self.assertFalse(os.path.isdir(tmpdir))

    def testDoNotCleanup(self):
        # test that cleanup cleans up
        tmpdir = tempfile.mkdtemp()
        self.hh = hhblits3.HHblits('testfiles/seq2.fasta', self.hhroot,
                                   working_dir=tmpdir)
        self.assertTrue(os.path.isdir(tmpdir))
        self.hh.needs_cleanup = False
        self.hh.Cleanup()
        self.assertTrue(os.path.isdir(tmpdir))
        shutil.rmtree(tmpdir)
        self.assertFalse(os.path.isdir(tmpdir))

    def testCleanupFailed(self):
        # test that cleanup cleans up
        tmpdir = tempfile.mkdtemp()
        self.hh = hhblits3.HHblits('testfiles/seq2.fasta', self.hhroot,
                                   working_dir=tmpdir)
        self.assertTrue(os.path.isdir(tmpdir))
        self.hh.needs_cleanup = False
        self.hh.CleanupFailed()
        self.assertFalse(os.path.isdir(tmpdir))

    def testFileNoExists(self):
        # test on calling with a non-existing fasta file
        with self.assertRaises(IOError) as ioe:
            hhblits3.HHblits('notthere.fasta', self.hhroot)
        self.assertEqual(ioe.exception.errno, 2)
        self.assertEqual(ioe.exception.strerror, 'No such file or directory')

    def testProfile(self):
        # try to create a search profile
        query_seq = seq.CreateSequence('Test', 'VLSPADKTNVKAAWGKVGAHAGEYGAEA'+
                                       'LERMFLSFPTTKTYFPHFDLSHGSAQVKGHGKKVAD'+
                                       'ALTNAVAHVDDMPNALSALSDLHAHKLRVDPVNFKL'+
                                       'LSHCLLVTLAAHLPAEFTPAVHASLDKFLASVSTVL'+
                                       'TSKYR')
        self.hh = hhblits3.HHblits(query_seq, self.hhroot)
        a3m = self.hh.BuildQueryMSA('testfiles/hhblitsdb/unittestdb', 
                                    assign_ss = False)
        self.assertTrue(filecmp.cmp(a3m, "testfiles/testali_two_no_ss.a3m"))

    def testA3mToProfileFileName(self):
        # test A3mToProfile to work with a given hhmake_file name
        query_seq = seq.CreateSequence('Test', 'VLSPADKTNVKAAWGKVGAHAGEYGAEA'+
                                       'LERMFLSFPTTKTYFPHFDLSHGSAQVKGHGKKVAD'+
                                       'ALTNAVAHVDDMPNALSALSDLHAHKLRVDPVNFKL'+
                                       'LSHCLLVTLAAHLPAEFTPAVHASLDKFLASVSTVL'+
                                       'TSKYR')
        self.hh = hhblits3.HHblits(query_seq, self.hhroot)
        _, self.tmpfile = tempfile.mkstemp(suffix='.hmm')
        os.remove(self.tmpfile)
        hhfile = self.hh.A3MToProfile("testfiles/testali.a3m",
                                      hhm_file=self.tmpfile)
        with open(hhfile) as tfh:
          tlst = tfh.readlines()
        with open("testfiles/test.hmm") as efh:
          elst = efh.readlines()
        self.assertEqual(len(elst), len(tlst))
        for i in range(0, len(elst)):
            if not elst[i].startswith(('FILE', 'COM', 'DATE')):
                self.assertEqual(elst[i], tlst[i])

    def testA3mToProfileWithoutFileName(self):
        # test A3mToProfile to work without a given hhmake_file name
        query_seq = seq.CreateSequence('Test', 'VLSPADKTNVKAAWGKVGAHAGEYGAEA'+
                                       'LERMFLSFPTTKTYFPHFDLSHGSAQVKGHGKKVAD'+
                                       'ALTNAVAHVDDMPNALSALSDLHAHKLRVDPVNFKL'+
                                       'LSHCLLVTLAAHLPAEFTPAVHASLDKFLASVSTVL'+
                                       'TSKYR')
        self.hh = hhblits3.HHblits(query_seq, self.hhroot)
        hhfile = self.hh.A3MToProfile("testfiles/testali.a3m")
        with open(hhfile) as tfh:
          tlst = tfh.readlines()
        with open("testfiles/test.hmm") as efh:
          elst = efh.readlines()
        self.assertEqual(len(elst), len(tlst))
        for i in range(0, len(elst)):
            if not elst[i].startswith(('FILE', 'COM', 'DATE')):
                self.assertEqual(elst[i], tlst[i])
        os.remove(hhfile)

    def testA3mToProfileWithExistingFile(self):
        # test A3mToProfile to work with an existing hmm file
        query_seq = seq.CreateSequence('Test', 'VLSPADKTNVKAAWGKVGAHAGEYGAEA'+
                                       'LERMFLSFPTTKTYFPHFDLSHGSAQVKGHGKKVAD'+
                                       'ALTNAVAHVDDMPNALSALSDLHAHKLRVDPVNFKL'+
                                       'LSHCLLVTLAAHLPAEFTPAVHASLDKFLASVSTVL'+
                                       'TSKYR')
        self.hh = hhblits3.HHblits(query_seq, self.hhroot)
        hhfile = self.hh.A3MToProfile("testfiles/testali.a3m",
                                      hhm_file="testfiles/test.hmm")
        # when the hmm file already exists, its not touched, so files should be
        # completely equal.
        self.assertTrue(filecmp.cmp(hhfile, "testfiles/test.hmm"))

    def testA3mToProfileWithoutA3m(self):
        # test A3mToProfile to fail if we do not have an a3m file.
        query_seq = seq.CreateSequence('Test', 'VLSPADKTNVKAAWGKVGAHAGEYGAEA'+
                                       'LERMFLSFPTTKTYFPHFDLSHGSAQVKGHGKKVAD'+
                                       'ALTNAVAHVDDMPNALSALSDLHAHKLRVDPVNFKL'+
                                       'LSHCLLVTLAAHLPAEFTPAVHASLDKFLASVSTVL'+
                                       'TSKYR')
        self.hh = hhblits3.HHblits(query_seq, self.hhroot)
        with self.assertRaises(IOError) as ioe:
            self.hh.A3MToProfile("doesnotexist.a3m")
        self.assertEqual(ioe.exception.errno, None)
        self.assertEqual(ioe.exception.args[0],
                         "could not convert a3m to hhm file")

    def testA3mToCSFileName(self):
        # test A3mToCS to work with a given hhmake_file name
        query_seq = seq.CreateSequence('Test', 'VLSPADKTNVKAAWGKVGAHAGEYGAEA'+
                                       'LERMFLSFPTTKTYFPHFDLSHGSAQVKGHGKKVAD'+
                                       'ALTNAVAHVDDMPNALSALSDLHAHKLRVDPVNFKL'+
                                       'LSHCLLVTLAAHLPAEFTPAVHASLDKFLASVSTVL'+
                                       'TSKYR')
        self.hh = hhblits3.HHblits(query_seq, self.hhroot)
        _, self.tmpfile = tempfile.mkstemp(suffix='.seq219')
        os.remove(self.tmpfile)
        csfile = self.hh.A3MToCS("testfiles/testali.a3m",
                                 cs_file=self.tmpfile, options={'-alphabet' :
                                                os.path.join(self.hhroot,
                                                             'data',
                                                             'cs219.lib')})
        self.assertTrue(filecmp.cmp(csfile, 'testfiles/test.seq219'))

    def testA3mToCSNoFileName(self):
        # test A3mToCS to work with a given hhmake_file name
        query_seq = seq.CreateSequence('Test', 'VLSPADKTNVKAAWGKVGAHAGEYGAEA'+
                                       'LERMFLSFPTTKTYFPHFDLSHGSAQVKGHGKKVAD'+
                                       'ALTNAVAHVDDMPNALSALSDLHAHKLRVDPVNFKL'+
                                       'LSHCLLVTLAAHLPAEFTPAVHASLDKFLASVSTVL'+
                                       'TSKYR')
        self.hh = hhblits3.HHblits(query_seq, self.hhroot)

        csfile = self.hh.A3MToCS("testfiles/testali.a3m",
                                 options={'-alphabet' :
                                          os.path.join(self.hhroot,
                                                       'data',
                                                       'cs219.lib')})
        self.assertTrue(filecmp.cmp(csfile, 'testfiles/test.seq219'))
        self.tmpfile = "testfiles/testali.seq219"

    def testA3mToCSFileExists(self):
        # test A3mToCS to work with a given hhmake_file name
        query_seq = seq.CreateSequence('Test', 'VLSPADKTNVKAAWGKVGAHAGEYGAEA'+
                                       'LERMFLSFPTTKTYFPHFDLSHGSAQVKGHGKKVAD'+
                                       'ALTNAVAHVDDMPNALSALSDLHAHKLRVDPVNFKL'+
                                       'LSHCLLVTLAAHLPAEFTPAVHASLDKFLASVSTVL'+
                                       'TSKYR')
        self.hh = hhblits3.HHblits(query_seq, self.hhroot)
        csfile = self.hh.A3MToCS("testfiles/testali.a3m",
                                 cs_file='testfiles/test.seq219',
                                 options={'-alphabet' :
                                          os.path.join(self.hhroot,
                                                       'data',
                                                       'cs219.lib')})
        self.assertTrue(filecmp.cmp(csfile, 'testfiles/test.seq219'))

    def testSearchWorking(self):
        # successful search
        query_seq = seq.CreateSequence('Test', 'VLSPADKTNVKAAWGKVGAHAGEYGAEA'+
                                       'LERMFLSFPTTKTYFPHFDLSHGSAQVKGHGKKVAD'+
                                       'ALTNAVAHVDDMPNALSALSDLHAHKLRVDPVNFKL'+
                                       'LSHCLLVTLAAHLPAEFTPAVHASLDKFLASVSTVL'+
                                       'TSKYR')
        self.hh = hhblits3.HHblits(query_seq, self.hhroot)
        search_file = self.hh.Search("testfiles/testali.a3m",
                                     'testfiles/hhblitsdb/unittestdb')

        with open(search_file) as f:
              header, hits = hhblits3.ParseHHblitsOutput(f)
        self.assertEqual(len(hits), 4)
        exp_scores = [199.3, 12.3, 11.5, 8.1]
        for hit_idx in range(4):
            self.assertAlmostEqual(hits[hit_idx].score, exp_scores[hit_idx])

    def testSearchNotWorking(self):
        query_seq = seq.CreateSequence('Test', 'VLSPADKTNVKAAWGKVGAHAGEYGAEA'+
                                       'LERMFLSFPTTKTYFPHFDLSHGSAQVKGHGKKVAD'+
                                       'ALTNAVAHVDDMPNALSALSDLHAHKLRVDPVNFKL'+
                                       'LSHCLLVTLAAHLPAEFTPAVHASLDKFLASVSTVL'+
                                       'TSKYR')
        self.hh = hhblits3.HHblits(query_seq, self.hhroot)
        with self.assertRaises(RuntimeError) as rte:
            self.hh.Search("doesnotexist.a3m", 'testfiles/hhblitsdb/unittestdb')

    def testParseHHMWorking(self):
        # get info from an HHM file
        with open("testfiles/test.hmm") as hhm_fh:
            prof = hhblits3.ParseHHM(hhm_fh)
        self.assertEqual(''.join([str(x) for x in prof['ss_conf']]),
                         '999999999999998873391557999999998639441123987788888'+
                         '856788999999999998735477789999999887650299989899889'+
                         '999999997536679989999999999999999984329')
        self.assertEqual(''.join(prof['ss_pred']), 'CCCHHHHHHHHHHHHHHHCCCHHHH'+
                         'HHHHHHHHHHCCCCCCCCCCCCCCCCCHHHHHHHHHHHHHHHHHHHCCCCH'+
                         'HHHHHHHHHHHHHHCCCCHHHHHHHHHHHHHHHHHHCCCCCCHHHHHHHHH'+
                         'HHHHHHHHHHHHCC')
        self.assertEqual(str(prof['consensus']), 'xltxxxxxxixxsWxxvxxxxxxxgxx'+
                         'xxxxlfxxxPxxxxxFxxxxxxxxxxxxxxhxxxvxxxlxxxixxldxxxx'+
                         'xlxxlxxxHxxxxgvxxxxxxxxxxxlxxxlxxxxgxxxxxxxxxAWxxxx'+
                         'xxixxxmxxxyx')
        self.assertEqual(prof['msa'].GetCount(), 7)

    def testParseHHMNotWorking(self):
        # get info from an HHM file
        with self.assertRaises(IOError) as ioe:
            with open('testfiles/testali.a3m') as f:
              hhblits3.ParseHHM(f)
        self.assertEqual(ioe.exception.args[0],
                         'Profile file "testfiles/testali.a3m" is missing '+
                         'the "Consensus" section')

    def testParseA3MWorking(self):
        # get info from an HHM file
        with open("testfiles/testali.a3m") as a3m_fh:
            prof = hhblits3.ParseA3M(a3m_fh)
        self.assertEqual(''.join([str(x) for x in prof['ss_conf']]),
                         '999999999999998873391557999999998639441123987788888'+
                         '856788999999999998735477789999999887650299989899889'+
                         '999999997536679989999999999999999984329')
        self.assertEqual(''.join(prof['ss_pred']), 'CCCHHHHHHHHHHHHHHHCCCHHHH'+
                         'HHHHHHHHHHCCCCCCCCCCCCCCCCCHHHHHHHHHHHHHHHHHHHCCCCH'+
                         'HHHHHHHHHHHHHHCCCCHHHHHHHHHHHHHHHHHHCCCCCCHHHHHHHHH'+
                         'HHHHHHHHHHHHCC')
        self.assertEqual(prof['msa'].GetCount(), 253)

    def fastParseHeader(self):
        header_line = '  1 814cbc1899f35c872169524af30fc2 100.0  5E-100' + \
                      '  5E-104  710.5  34.1  277    3-293     2-280 (281)'
        hit, offset = hhblits3.ParseHeaderLine(header_line)
        self.assertEqual(hit.hit_id, '814cbc1899f35c872169524af30fc2')
        self.assertAlmostEqual(hit.evalue, 0)
        self.assertAlmostEqual(hit.prob, 100.0)
        self.assertAlmostEqual(hit.pvalue, 0)
        self.assertAlmostEqual(hit.score, 710.5)
        self.assertAlmostEqual(hit.ss_score, 34.1)

    def testParseHHblitsOutput(self):
        with open("testfiles/test.hhr") as f:
          header, hits = hhblits3.ParseHHblitsOutput(f)
        self.assertEqual(header.query, 'Test')
        self.assertEqual(header.match_columns, 141)
        self.assertEqual(header.n_eff, 9.4)
        self.assertEqual(header.searched_hmms, 5)
        self.assertEqual(header.date,
                         datetime.datetime.strptime('Mon Aug 31 16:45:45 2015',
                                                    '%a %b %d %H:%M:%S %Y'))
        self.assertEqual(header.command, '/import/bc2/apps/HH-suite/2.0.16-go'+
                         'olf-1.4.10/bin/hhblits -cpu 1 -n 1 -e 0.001 -Z 1000'+
                         '0 -B 10000 -i /import/bc2/home/schwede/bienert/git/'+
                         'ost_newenv.git/modules/bindings/tests/testfiles/tes'+
                         'tali.a3m -o /scratch/tmp9E5dor/testali_cpu1_n1.hhr '+
                         '-d /import/bc2/home/schwede/bienert/git/ost_newenv.'+
                         'git/modules/bindings/tests/testfiles/hhblitsdb/hhbl'+
                         'itsdb')
        self.assertEqual(len(hits), 4)
        # hit 1
        self.assertEqual(hits[0].hit_id, '3e7b90809bd446a538f9eb1a1ca0e551')
        self.assertEqual(hits[0].score, float('222.3'))
        self.assertEqual(hits[0].ss_score, float('16.4'))
        self.assertEqual(hits[0].evalue, float('2.7e-42'))
        self.assertEqual(hits[0].pvalue, float('6.9E-45'))
        self.assertEqual(hits[0].prob, float('100.0'))
        self.assertEqual(str(hits[0].aln),
                         'Test                VLSPADKTNVKAAWGKVGAHAGEYGAEALER'+
                         'MFLSFPTTKTYFPHF-DL-S----HGSAQ\n3e7b90809bd446a5... '+
                         'VLSEGEWQLVLHVWAKVEADVAGHGQDILIRLFKSHPETLEKFDRFKHLKT'+
                         'EAEMKASED\n\nTest                VKGHGKKVADALTNAVAH'+
                         'VDDMPNALSALSDLHAHK-LRVDPVNFKLLSHCLLVTLAAHL\n3e7b908'+
                         '09bd446a5... LKKHGVTVLTALGAILKKKGHHEAELKPLAQSHA-TKH'+
                         'KIPIKYLEFISEAIIHVLHSRH\n\nTest                PAEFT'+
                         'PAVHASLDKFLASVSTVLTSKYR\n3e7b90809bd446a5... PGDFGA'+
                         'DAQGAMNKALELFRKDIAAKYK\n')
        # hit 2
        self.assertEqual(hits[1].hit_id, 'af828e69a5f2d0fd42a2a213e09eb64f')
        self.assertEqual(hits[1].score, float('215.7'))
        self.assertEqual(hits[1].ss_score, float('17.2'))
        self.assertEqual(hits[1].evalue, float('1.5e-41'))
        self.assertEqual(hits[1].pvalue, float('3.9E-44'))
        self.assertEqual(hits[1].prob, float('100.0'))
        self.assertEqual(str(hits[1].aln),
                         'Test                VLSPADKTNVKAAWGKVGAHAGEYGAEALER'+
                         'MFLSFPTTKTYFPHFDLSHGSAQVKGHGK\naf828e69a5f2d0fd... '+
                         'VLSPADKTNVKAAWGKVGAHAGEYGAEALERMFLSFPTTKTYFPHFDLSHG'+
                         'SAQVKGHGK\n\nTest                KVADALTNAVAHVDDMPN'+
                         'ALSALSDLHAHKLRVDPVNFKLLSHCLLVTLAAHLPAEFTPA\naf828e6'+
                         '9a5f2d0fd... KVADALTNAVAHVDDMPNALSALSDLHAHKLRVDPVNF'+
                         'KLLSHCLLVTLAAHLPAEFTPA\n\nTest                VHASL'+
                         'DKFLASVSTVLTSKYR\naf828e69a5f2d0fd... VHASLDKFLASVS'+
                         'TVLTSKYR\n')
        # hit 3
        self.assertEqual(hits[2].hit_id, '9287755aa6aa27583da6be3b2408bfcc')
        self.assertEqual(hits[2].score, float('215.1'))
        self.assertEqual(hits[2].ss_score, float('16.5'))
        self.assertEqual(hits[2].evalue, float('3.8e-41'))
        self.assertEqual(hits[2].pvalue, float('9.7E-44'))
        self.assertEqual(hits[2].prob, float('100.0'))
        self.assertEqual(str(hits[2].aln),
                         'Test                VLSPADKTNVKAAWGKVGAHAGEYGAEALER'+
                         'MFLSFPTTKTYFPHF-DLS-----HGSAQ\n9287755aa6aa2758... '+
                         'HLTPEEKSAVTALWGKVN--VDEVGGEALGRLLVVYPWTQRFFESFGDLST'+
                         'PDAVMGNPK\n\nTest                VKGHGKKVADALTNAVAH'+
                         'VDDMPNALSALSDLHAHK-LRVDPVNFKLLSHCLLVTLAAHL\n9287755'+
                         'aa6aa2758... VKAHGKKVLGAFSDGLAHLDNLKGTFATLSELHC-DKL'+
                         'HVDPENFRLLGNVLVCVLAHHF\n\nTest                PAEFT'+
                         'PAVHASLDKFLASVSTVLTSKYR\n9287755aa6aa2758... GKEFTP'+
                         'PVQAAYQKVVAGVANALAHKYH\n')
        # hit 4
        self.assertEqual(hits[3].hit_id, 'e69e1ac0a4b2554df25f2c2183b0fba0')
        self.assertEqual(hits[3].score, float('12.6'))
        self.assertEqual(hits[3].ss_score, float('2.6'))
        self.assertEqual(hits[3].evalue, float('64.0'))
        self.assertEqual(hits[3].pvalue, float('0.16'))
        self.assertEqual(hits[3].prob, float('0.6'))
        self.assertEqual(str(hits[3].aln),
                         'Test                VDPVNFKLLSHCLLVTLAAHL\ne69e1ac0'+
                         'a4b2554d... ATPEQAQLVHKEIRKIVKDTC\n')

if __name__ == "__main__":
    try:
        from ost.bindings import hhblits
        hhblits_version_string = hhblits.GetHHblitsVersionString()
    except:
        print("No HHblits installation found: skip unit tests")
        sys.exit(0)
    if hhblits_version_string is None:
        print("Could not identify HHblits version: skip unit tests")
        sys.exit(0)
    hhblits_version_major = int(hhblits_version_string.split('.')[0])
    if hhblits_version_major != 3:
        print("Found hhblits with major version %i. Unit tests only run with "\
              "version 3: skip unit tests"%(hhblits_version_major))
        sys.exit(0)

    from ost import testutils
    testutils.RunTests()



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------

# Functions to use Antechamber (from AmberTools) to automatically generate force
# field parameters. Allows the execution of Antechamber and the parsing of files
# generated by it.

from ost.mol import mm
import ost
from ost import settings, mol, geom
import os, subprocess, math

###############################################################################
# helper functions
def _GetInteraction(functype, atoms, params):
  """Get an mm.Interaction with the given func-type and params for the given
  atoms (name and types extracted from there)."""
  interaction = mm.Interaction(functype)
  interaction.SetNames([a.name for a in atoms])
  interaction.SetTypes([a.GetStringProp('type') for a in atoms])
  interaction.SetParam(params)
  return interaction

def _MakeComponentBuildingBlock(eh, ff_dict):
  """Take EntityHandle eh (from ParseModifiedPDB) and ff_dict (from 
  ParseAmberForceField) and return BuildingBlock."""
  # converters: length: A -> nm, angles: deg -> rad, energy: kcal -> kJ
  dist_scale = 1./10.0
  angle_scale = math.pi/180.
  # bond strength in OpenMM needs a factor of 2 compared to Amber
  bond_k_scale = 418.4*2.
  angle_k_scale = 4.184
  
  # get atom typing (dictionary listing all atoms of a certain type)
  atype_dict = {}
  for a in eh.atoms:
    atype = a.GetStringProp('type')
    if not atype in atype_dict:
      atype_dict[atype] = [a.handle]
    else:
      atype_dict[atype].append(a.handle)
  
  # set masses in entity handle (charges and types set in ParseModifiedPDB)
  for atype, mass in ff_dict['MASS']:
    for a in atype_dict[atype]: a.SetMass(mass)
  
  # start by adding atoms
  bb = mm.BuildingBlock()
  for a in eh.atoms:
    bb.AddAtom(a.name, a.GetStringProp('type'), a.GetCharge(), a.GetMass())
  
  # add bonds: first all bonds from the entity, then force-field params
  bl = eh.GetBondList()
  for b in bl:
    a1 = b.GetFirst()
    a2 = b.GetSecond()
    bond = mm.Interaction(mm.FuncType.HarmonicBond)
    bond.SetNames([a1.name, a2.name])
    bond.SetTypes([a1.GetStringProp('type'), a2.GetStringProp('type')])
    bb.AddBond(bond)
  added_bonds = []
  for atype1, atype2, d0, k in ff_dict['BOND']:
    for a1 in atype_dict[atype1]:
      for a2 in atype_dict[atype2]:
        # check connectivity and uniqueness of bond
        if not mol.BondExists(a1, a2): continue
        if [a1, a2] in added_bonds or [a2, a1] in added_bonds: continue
        added_bonds.append([a1,a2])
        params = [d0*dist_scale, k*bond_k_scale]
        bond = _GetInteraction(mm.FuncType.HarmonicBond, [a1, a2], params)
        bb.AddBond(bond, replace_existing=True)
  
  # add angles
  added_angles = []
  for atype1, atype2, atype3, a0, k in ff_dict['ANGL']:
    # a2 is the central atom
    for a2 in atype_dict[atype2]:
      for a1 in atype_dict[atype1]:
        if not mol.BondExists(a1, a2): continue
        for a3 in atype_dict[atype3]:
          # check connectivity and uniqueness of angle
          if not mol.BondExists(a2, a3): continue
          if a1 == a3: continue
          if [a1, a2, a3] in added_angles or [a3, a2, a1] in added_angles:
            continue
          added_angles.append([a1, a2, a3])
          angle = _GetInteraction(mm.FuncType.HarmonicAngle, [a1, a2, a3],
                                  [a0*angle_scale, k*angle_k_scale*2])
          bb.AddAngle(angle)
  
  # add dihedrals
  for atype1, atype2, atype3, atype4, idiv, period, phase, k in ff_dict['DIHE']:
    # there can be multiple ones for the same set of types!
    added_dihedrals = []
    for a1 in atype_dict[atype1]:
      for a2 in atype_dict[atype2]:
        if not mol.BondExists(a1, a2): continue
        for a3 in atype_dict[atype3]:
          if not mol.BondExists(a2, a3): continue
          if a1 == a3: continue
          for a4 in atype_dict[atype4]:
            # check connectivity and uniqueness of dihedral (can be mirrored)
            if not mol.BondExists(a3, a4): continue
            if a2 == a4: continue
            if [a1, a2, a3, a4] in added_dihedrals or \
               [a4, a3, a2, a1] in added_dihedrals: continue
            added_dihedrals.append([a1, a2, a3, a4])
            dihe = _GetInteraction(mm.FuncType.PeriodicDihedral, [a1, a2, a3, a4],
                                   [period, phase*angle_scale, k*angle_k_scale])
            bb.AddDihedral(dihe)
  
  # add impropers
  added_impropers = []
  for atype1, atype2, atype3, atype4, period, phase, k in ff_dict['IMPR']:
    # third atom is the central atom in amber force-field
    for ac in atype_dict[atype3]:
      for a1 in atype_dict[atype1]:
        if not mol.BondExists(ac, a1): continue
        for a2 in atype_dict[atype2]:
          if not mol.BondExists(ac, a2): continue
          if a1 == a2: continue
          for a4 in atype_dict[atype4]:
            # check connectivity and uniqueness of impr. (same central)
            if not mol.BondExists(ac, a4): continue
            if a2 == a4 or a1 == a4: continue
            if [ac, a1, a2, a4] in added_impropers or \
               [ac, a1, a4, a2] in added_impropers or \
               [ac, a2, a1, a4] in added_impropers or \
               [ac, a2, a4, a1] in added_impropers or \
               [ac, a4, a1, a2] in added_impropers or \
               [ac, a4, a2, a1] in added_impropers: continue
            added_impropers.append([ac, a1, a2, a4])
            impr = _GetInteraction(mm.FuncType.PeriodicImproper, [a1, a2, ac, a4],
                                   [period, phase*angle_scale, k*angle_k_scale])
            bb.AddImproper(impr)  
  return bb

def _ParseModifiedPDB(filename):
  """Read mpdb file produced by antechamber and return tuple of:
  - EntityHandle with connectivity, atom types (property 'type') and charges
  - Residue name as extracted from the mpdb file
  A RuntimeError is raised if the file can contains multiple residues.
  """
  eh = mol.CreateEntity()
  rname = ''
  edi = eh.EditXCS(mol.BUFFERED_EDIT)
  chain = edi.InsertChain('A')
  bond_list = []
  # get all atoms and bonds from file
  with open(filename, 'r') as in_file:
    for line in in_file:
      # atom or connectivity
      # -> fixed column format assumed for both
      if line.startswith('ATOM'):
        aname = line[12:17].strip()
        # extract res. name and ensure uniqueness
        if not rname:
          rname = line[17:20].strip()
          r = edi.AppendResidue(chain, rname)
        elif rname != line[17:20].strip():
          raise RuntimeError("More than one residue in file " + filename +\
                             ". Cannot parse!")
        # extract and store type and charge
        charge = float(line[54:66])
        atype = line[78:80].strip()
        a = edi.InsertAtom(r, aname, geom.Vec3())
        a.SetStringProp('type', atype)
        a.SetCharge(charge)
      elif line.startswith('CONECT'):
        ai1 = int(line[6:11])
        # max. 4 bond partners...
        for i in range(4):
          try:
            j = 11 + 5*i
            ai2 = int(line[j:j+5])
            # only unique bonds added to list
            s = set([ai1, ai2])
            if not s in bond_list: bond_list.append(s)
          except:
            # exception thrown for empty strings or non-integers
            # -> skip
            continue
  # set all bonds in entity
  for indices in bond_list:
    indices = list(indices)
    a1 = r.atoms[indices[0]-1]
    a2 = r.atoms[indices[1]-1]
    edi.Connect(a1, a2)
  # finalize
  edi.UpdateICS()
  return eh, rname

def _ParseAmberForceField(filename):
  """Read frcmod file produced by parmchk2 and return dictionary with all
  entries for masses, bonds, angles, dihedrals, impropers and non-bonded (LJ)
  interactions. Stored as key/list-of-value pairs:
  - 'MASS': [atype, mass]
  - 'BOND': [atype1, atype2, d0, k]
  - 'ANGL': [atype1, atype2, atype3, a0, k]
  - 'DIHE': [atype1, atype2, atype3, atype4, idiv, period, phase, k/idiv]
  - 'IMPR': [atype1, atype2, atype3, atype4, period, phase, k]
  - 'NONB': [Rvdw, epsilon]
  """
  keywords = ['MASS', 'BOND', 'ANGL', 'DIHE', 'IMPR', 'NONB']
  with open(filename, 'r') as in_file:
    ff_dict = {}
    for line in in_file:
      # look for keywords
      keyword = line[:4]
      if not keyword in keywords: continue
      # loop until empty line found
      ff_dict[keyword] = []
      line = next(in_file)
      while len(line.strip()) > 0:
        # check for warnings
        if 'ATTN' in line:
          ost.LogWarning('The following line in ' + filename + ' (' + keyword +\
                         ' section) needs revision:\n' + line.strip())
        # fixed column format -> extract entries dep. on current keyword
        if keyword == 'MASS':
          atype = line[0:2].strip()
          s = line[2:].split()
          mass = float(s[0])
          ff_dict[keyword].append([atype, mass])
        elif keyword == 'BOND':
          atype1 = line[:2].strip()
          atype2 = line[3:5].strip()
          s = line[5:].split()
          k = float(s[0])
          d0 = float(s[1])
          ff_dict[keyword].append([atype1, atype2, d0, k])
        elif keyword == 'ANGL':
          atype1 = line[:2].strip()
          atype2 = line[3:5].strip()
          atype3 = line[6:8].strip()
          s = line[8:].split()
          k = float(s[0])
          a0 = float(s[1])
          ff_dict[keyword].append([atype1, atype2, atype3, a0, k])
        elif keyword == 'DIHE':
          atype1 = line[:2].strip()
          atype2 = line[3:5].strip()
          atype3 = line[6:8].strip()
          atype4 = line[9:11].strip()
          s = line[11:].split()
          idiv = float(s[0])
          k = float(s[1])
          phase = float(s[2])
          # negative periods = there is more than one term for that dihedral
          # -> no need to do anything special about that here...
          period = abs(float(s[3]))
          ff_dict[keyword].append([atype1, atype2, atype3, atype4, idiv,
                                   period, phase, k/float(idiv)])
        elif keyword == 'IMPR':
          atype1 = line[:2].strip()
          atype2 = line[3:5].strip()
          atype3 = line[6:8].strip()
          atype4 = line[9:11].strip()
          s = line[11:].split()
          k = float(s[0])
          phase = float(s[1])
          period = float(s[2])
          ff_dict[keyword].append([atype1, atype2, atype3, atype4, period,
                                   phase, k])
        elif keyword == 'NONB':
          line = line.strip()
          atype = line[0:2].strip()
          s = line[2:].split()
          Rvdw = float(s[0])
          epsilon = float(s[1])
          ff_dict[keyword].append([atype, Rvdw, epsilon])
        # next...
        line = next(in_file)
  return ff_dict
###############################################################################

def RunAntechamber(res_name, filename, format='ccif', amberhome=None,
                   base_out_dir=None):
  """Run Antechamber to guess force field parameters for a given residue name.

  This requires an installation of AmberTools (tested with AmberTools15) with
  binaries ``antechamber`` and ``parmchk2``.

  This has the same restrictions as Antechamber itself and we assume the input
  to be uncharged. Note that Antechamber cannot deal with metal ions and other
  non-organic elements.

  The results are stored in a separate folder named `res_name` within
  `base_out_dir` (if given, otherwise the current working directory). The main
  output files are ``frcmod`` and ``out.mpdb``. The former contains force field
  parameters and masses. The latter maps atom names to atom types and defines
  the partial charges. The same output could be obtained as follows:

  .. code-block:: console

     $ antechamber -i <FILENAME> -fi <FORMAT> -bk '<RES_NAME>' -o out.mol2 -fo mol2 -c bcc -pf yes
     $ parmchk2 -i out.mol2 -f mol2 -o frcmod -a Y
     $ antechamber -i out.mol2 -fi mol2 -o out.mpdb -fo mpdb -pf yes

  The force field parameters can be manually modified if needed. It can for
  instance happen that some parameters cannot be identified. Those lines will
  be marked with a comment "ATTN, need revision".

  :param res_name: Residue name for which we desire force field parameters.
  :type res_name:  :class:`str`
  :param filename: Path to a file which contains the necessary information for
                   `res_name`. It must include all hydrogens.
  :type filename:  :class:`str`
  :param format: Format of file given with `filename`. Common formats are 'ccif'
                 for PDB's component dictionary or 'pdb' for a PDB file
                 containing the desired residue with all hydrogens.
  :type format:  :class:`str`
  :param amberhome: Base path of your AmberTools installation. If not None,
                    we look for ``antechamber`` and ``parmchk2`` within
                    ``AMBERHOME/bin`` additionally to the system's ``PATH``.
  :type amberhome:  :class:`str`
  :param base_out_dir: Path to a base path, where the output will be stored.
                       If None, the current working directory is used.
  :type base_out_dir:  :class:`str`
  """
  # find antechamber binaries
  if amberhome is None:
    search_paths = []
  else:
    search_paths = [os.path.join(amberhome, 'bin')]
  try:
    antechamber = settings.Locate('antechamber', search_paths=search_paths)
    parmchk2 = settings.Locate('parmchk2', search_paths=search_paths)
  except settings.FileNotFound as ex:
    ost.LogError("Failed to find Antechamber binaries. Make sure you have "
                 "AmberTools installed!")
    raise ex
  
  # prepare path
  cwd = os.getcwd()
  if base_out_dir is None:
    base_out_dir = cwd
  out_dir = os.path.abspath(os.path.join(base_out_dir, res_name))
  if not os.path.exists(out_dir):
    # note: this creates intermediate folders too
    try:
      os.makedirs(out_dir)
    except Exception as ex:
      ost.LogError("Failed to create output directory " + out_dir + "!")
      raise ex
  
  # execute it
  os.chdir(out_dir)
  try:
    cmds = [antechamber + " -i " + filename + " -fi " + format + " -bk " \
            + res_name + " -o out.mol2 -fo mol2 -c bcc -pf yes",
            parmchk2 + " -i out.mol2 -f mol2 -o frcmod -a Y",
            antechamber + " -i out.mol2 -fi mol2 -o out.mpdb -fo mpdb -pf yes"]
    all_sout = "Generating force field parameters for " + res_name + "\n"
    all_serr = ""
    for cmd in cmds:
      all_sout += "-"*70 + "\n" + "Stdout of: " + cmd + "\n" + "-"*70 + "\n"
      all_serr += "-"*70 + "\n" + "Stderr of: " + cmd + "\n" + "-"*70 + "\n"
      job = subprocess.Popen(cmd.split(" "), stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
      sout, serr = job.communicate()
      all_sout += sout
      all_serr += serr
      if job.returncode != 0:
        ost.LogError("Unsuccessful execution of " + cmd + ". Return code: "\
                     + str(job.returncode))
    # write command line outputs
    with open("console.stdout", "w") as txt_file:
      txt_file.write(all_sout)
    with open("console.stderr", "w") as txt_file:
      txt_file.write(all_serr)
  except Exception as ex:
    ost.LogError("Failed to excecute antechamber binaries!")
    raise ex
  
  # get back to original path
  os.chdir(cwd)

  # check result
  frcmod_filename = os.path.join(out_dir, 'frcmod')
  mpdb_filename = os.path.join(out_dir, 'out.mpdb')
  if not os.path.exists(frcmod_filename):
    raise RuntimeError("Failed to generate frcmod file with Antechamber!")
  if not os.path.exists(mpdb_filename):
    raise RuntimeError("Failed to generate out.mpdb file with Antechamber!")

def AddFromFiles(force_field, frcmod_filename, mpdb_filename):
  """Add data from a frcmod and an mpdb file to a force field.

  This will add a new :class:`~ost.mol.mm.BuildingBlock` to `force_field` for
  the residue defined in those files (residue name is extracted from the mpdb
  file which can only contain a single residue). Charges for each atom are
  extracted from the mpdb file. According to the frcmod file, an
  :class:`~ost.mol.mm.Interaction` is added for each bond, angle, dihedral and
  improper. Atom types with masses and non-bonded interactions are added to
  `force_field` as needed.

  :param force_field: A force field object to which the new parameters are
                      added.
  :type force_field:  :class:`~ost.mol.mm.Forcefield`
  :param frcmod_filename: Path to ``frcmod`` file as generated by ``parmchk2``.
  :type frcmod_filename:  :class:`str`
  :param mpdb_filename: Path to mpdb file as generated by ``antechamber``.
  :type mpdb_filename:  :class:`str`
  :return: The updated force field (same as `force_field`).
  :rtype:  :class:`~ost.mol.mm.Forcefield`
  """
  # check files
  if not os.path.exists(frcmod_filename):
    raise RuntimeError("Could not find frcmod file: " + frcmod_filename)
  if not os.path.exists(mpdb_filename):
    raise RuntimeError("Could not find mpdb file: " + mpdb_filename)
  # read in files
  try:
    eh, res_name = _ParseModifiedPDB(mpdb_filename)
  except Exception as ex:
    ost.LogError("Failed to parse mpdb file: " + mpdb_filename)
    raise ex
  try:
    ff_dict = _ParseAmberForceField(frcmod_filename)
  except Exception as ex:
    ost.LogError("Failed to parse frcmod file: " + frcmod_filename)
    raise ex
  ost.LogInfo("Adding force field for " + res_name)
  # add atoms to force field
  for aname, mass in ff_dict['MASS']:
    force_field.AddMass(aname, mass)
  # add LJs
  lj_sigma_scale = 2./10./2**(1./6.) # Rvdw to sigma in nm
  lj_epsilon_scale = 4.184           # kcal to kJ
  for aname, Rvdw, epsilon in ff_dict['NONB']:
    # fix 0,0 (from OpenMM's processAmberForceField.py)
    if Rvdw == 0 or epsilon == 0:
      Rvdw, epsilon = 1, 0
    lj = mm.Interaction(mm.FuncType.LJ)
    lj.SetTypes([aname])
    lj.SetParam([Rvdw*lj_sigma_scale, epsilon*lj_epsilon_scale])
    force_field.AddLJ(lj)
  # add building block
  bb = _MakeComponentBuildingBlock(eh, ff_dict)
  force_field.AddBuildingBlock(res_name, bb)

  return force_field

def AddFromPath(force_field, out_dir):
  """Add data from a directory created with :meth:`Run` to a force field.
  See :meth:`AddFromFiles` for details.

  :param force_field: A force field object to which the new parameters are
                      added.
  :type force_field:  :class:`~ost.mol.mm.Forcefield`
  :param out_dir: Output directory as created with :meth:`Run`. Must contain
                  files ``frcmod`` and ``out.mpdb``.
  :type out_dir:  :class:`str`
  :return: The updated force field (same as `force_field`).
  :rtype:  :class:`~ost.mol.mm.Forcefield`
  """
  frcmod_filename = os.path.join(out_dir, 'frcmod')
  mpdb_filename = os.path.join(out_dir, 'out.mpdb')
  return AddFromFiles(force_field, frcmod_filename, mpdb_filename)

def AddIon(force_field, res_name, atom_name, atom_mass, atom_charge, lj_sigma,
           lj_epsilon):
  """Add a single atom as an ion to a force field.

  Since Antechamber cannot deal with ions, you can add simple ones easily with
  this function. This adds a :class:`~ost.mol.mm.BuildingBlock` to `force_field`
  for the given residue name containing a single atom. The atom will have a type
  with the same name as the atom name and the given mass, charge and non-bonded
  (LJ) interaction parameters.

  :param force_field: A force field object to which the ion is added.
  :type force_field:  :class:`~ost.mol.mm.Forcefield`
  :param res_name: Residue name for the ion to be added.
  :type res_name:  :class:`str`
  :param atom_name: Atom name which is also used as atom type name.
  :type atom_name:  :class:`str`
  :param atom_mass: Mass of the atom.
  :type atom_mass:  :class:`float`
  :param atom_charge: Charge of the atom.
  :type atom_charge:  :class:`float`
  :param lj_sigma: The sigma parameter for the non-bonded LJ interaction.
  :type lj_sigma:  :class:`float` in nm
  :param lj_epsilon: The sigma parameter for the non-bonded LJ interaction.
  :type lj_epsilon:  :class:`float` in kJ/mol
  """
  # add mass (atom_type = atom_name)
  force_field.AddMass(atom_name, atom_mass)
  # add building block
  bb = mm.BuildingBlock()
  bb.AddAtom(atom_name, atom_name, atom_charge, atom_mass)
  force_field.AddBuildingBlock(res_name, bb)
  # add dummy LJ
  lj = mm.Interaction(mm.FuncType.LJ)
  lj.SetTypes([atom_name])
  lj.SetParam([lj_sigma, lj_epsilon])
  force_field.AddLJ(lj)

__all__ = ('RunAntechamber', 'AddFromFiles', 'AddFromPath', 'AddIon',)



import bundle
import deps
import sys
from optparse import OptionParser
import shutil
import os
p=OptionParser()
p.add_option('--bundle', action='store_true', default=False)
p.add_option('--no_rpath', action='store_true',
             default=False)
p.add_option('--macports_workaround', action='store_true', default=False)
p.add_option('--dmg', action='store_true', default=False)
p.add_option('--no-gui', action='store_true', default=False)
opts, args=p.parse_args()
deps.make_standalone('../../stage', 'standalone', True, 
                     opts.no_rpath, 
                     macports_workaround=opts.macports_workaround,
                     no_gui=opts.no_gui)
if os.path.exists('DNG.app'):
  shutil.rmtree('DNG.app')
if opts.no_gui:
  out_dir='ost-%s' % ost.VERSION
  if os.path.exists(out_dir):
    shutil.rmtree(out_dir)
  shutil.move('standalone', out_dir)
  sys.exit(0)
bundle.create_bundle('DNG', opts.bundle)
if opts.bundle:
  shutil.copytree('../../examples', 'DNG.app/Contents/examples')
  os.system('rm `find DNG.app/Contents/examples/ -name "*.pyc"` 2> /dev/null')
  os.system('rm -rf DNG.app/Contents/examples/code_fragments/')
  os.system('rm -rf DNG.app/Contents/examples/gfx/')
  if opts.dmg:
    os.system('rm -rf openstructure-%s.dmg' % ost.VERSION)
    os.system('hdiutil create -srcFolder DNG.app openstructure-%s.dmg' % ost.VERSION)



def RunTests():
  '''
  This function behaves as a custom TestLoader for python unittests.

  With no system arguments, the default unittest TestRunner is used.

  If the first system argument (sys.argv[1]) is set to 'xml', a XMLTestRunner
  is used, which produces a JUnit compatible XML output file. Within the current
  module, each function is identified which is a subclass of unittest.TestCase
  and for each TestCase, a test suite is executed, producing an individual
  output file for each TestCase. The output file has the name,
  'PYTEST-<TestCaseName>.xml'.

  Example of a Python testcase:

  .. code-block:: python

    import unittest

    class TestRenumber(unittest.TestCase):

      def setUp(self):
        # prepare stuff"
        pass

      def testSomeFunction(self):
        # do some asserts
        pass

    if __name__ == "__main__":
      from ost import testutils
      testutils.RunTests()

  '''
  import unittest
  import sys
  try:
    if len(sys.argv)>1 and sys.argv[1]=='xml':
      import inspect
      import types
      import __main__
      from ost import xmlrunner
      for name, obj in inspect.getmembers(__main__):
        if (isinstance(obj, type) and
                            issubclass(obj, unittest.TestCase)):
          suite = unittest.TestLoader().loadTestsFromTestCase(obj)
          stream = open('PYTEST-%s.xml'%name, 'w')
          xmlrunner.XMLTestRunner(stream).run(suite)
          stream.close()

    else:
      unittest.main()
  except Exception as e:
    print(e)


def SetDefaultCompoundLib():
  '''
  This function tries to ensure that a default compound library is set.
  When calling scripts with ``ost`` this is not needed, but since unit tests are
  called with ``python`` we need to ensure that it is set. The function is only
  expected to succeed (and return True) if ``COMPOUND_LIB`` was set when
  :ref:`configuring the compilation <cmake-flags>`.

  It tries the following:

  - get it with :func:`ost.conop.GetDefaultLib`
  - look for ``compounds.chemlib`` in ``$OST_ROOT/share/openstructure``
  - if ``OST_ROOT`` not set in the above, try to guess it based on the path of
    the ``conop`` module

  To use this check modify the :func:`RunTests` call to read:

  .. code-block:: python

    if __name__ == "__main__":
      from ost import testutils
      if testutils.SetDefaultCompoundLib():
        testutils.RunTests()
      else:
        print 'No compound library available. Ignoring test_XXX.py tests.'

  :return: True, if a compound library was found and set to be accessed with
           :func:`ost.conop.GetDefaultLib`. False otherwise.
  '''
  import os
  from ost import conop, GetSharedDataPath, SetPrefixPath
  # check if already there
  if conop.GetDefaultLib():
    return True
  else:
    # try to get the shared data path?
    try:
      shared_data_path = GetSharedDataPath()
    except Exception as e:
      SetPrefixPath(os.path.abspath(os.path.join(conop.__path__[0], os.pardir,
                                                 os.pardir, os.pardir,
                                                 os.pardir, os.pardir)))
      shared_data_path = GetSharedDataPath()
    # look for compounds.chemlib in there
    compound_lib_path = os.path.join(shared_data_path, 'compounds.chemlib')
    if os.path.exists(compound_lib_path):
      compound_lib = conop.CompoundLib.Load(compound_lib_path)
      conop.SetDefaultLib(compound_lib)
      return True
    else:
      return False



"""
Author: Marco Biasini
"""

import re
from ost import seq

class HHSearchHit:
  def __init__(self, summary, alignment):
    self.summary=summary
    self.alignment=alignment

class HitSummary:
  def __init__(self, pdb_id, chain, prob, e_value, query_start, query_end, 
               template_start, template_end):
   self.prob=prob
   self.pdb_id=pdb_id
   self.chain=chain
   self.e_value=e_value   
   self.query_start=query_start
   self.query_end=query_end
   self.template_start=template_start
   self.template_end=template_end

class HHSearchResult:
  """
  Read HHSearch result file. The result is stored in a list of hh search hits.
  
  Usage:
  
  result=HHSearchResult('output.hhr')
  for hit in result.hits:
    print hit.pdb_id, hit.chain
    print hit.alignment.ToString(80)
  """
  def __init__(self, filename, pipe_separated=False):
    self.pipe_separated=pipe_separated    
    self._Read(filename)
  def _Read(self, filename):
    ifile=open(filename)
    self._ReadHeader(ifile)
    summaries=self._ReadHitSummaries(ifile)
    self.hits=self._ReadHitDetails(ifile, summaries)
    
  def _ReadHeader(self, ifile):
    header_lines=[]
    for line in ifile:
      stripped_line=line.strip()
      if stripped_line=='':
        break
      header_lines.append(stripped_line)
    for header_line in header_lines:
      var, value=re.split('\s+', header_line, 1)
      setattr(self, var.lower(), value)
      
  def _ReadHitSummaries(self, ifile):
    summary_lines=[]
    skip_header=True
    for line in ifile:
      if skip_header==True:
        skip_header=False
        continue

      stripped_line=line.strip()
      if stripped_line=='':
        break
      summary_lines.append(line)
    summaries=[]
    for summary_line in summary_lines:
      pdb_id, chain=(None, None)
      if self.pipe_separated:
        parts=summary_line[4:37].split('|')
        pdb_id=parts[1][:4]
        chain=parts[1][4]
      else:
        pdb_id=summary_line[4:8]
        chain=summary_line[9]
      prob=float(summary_line[36:40])
      e_value=0.0
      query_range=summary_line[76:84].split('-')
      query_start=int(query_range[0].strip())
      query_end=int(query_range[1].strip())
      template_range=summary_line[86:94].split('-')
      template_start=int(template_range[0].strip())
      template_end=int(template_range[1].strip())
      summaries.append(HitSummary(pdb_id, chain, prob, e_value, query_start,
                                  query_end, template_start, template_end))
    return summaries

  def _ReadHitDetails(self, ifile, summaries):
    hits=[]
    for summary in summaries:
      alignment=self._ReadHitDetail(ifile)
      hits.append(HHSearchHit(summary, alignment))
    return hits
  def _ReadHitDetail(self, ifile):
    skip_header=True
    q_seq, t_seq=('', '')
    for line in ifile:
      if skip_header==True:
        if line.startswith('>'):
          skip_header=False
        continue
      if line.startswith('No'):
        break
      if line.strip()=='':
        continue        
      if line.startswith('Q ss_pred') or line.startswith('Q Consensus'):
        continue
      if line.startswith('T ss_pred') or line.startswith('T Consensus'):
        continue
      if line.startswith(' '):
        continue
      if line.startswith('Q'):
        q_seq+=re.split('\s+', line)[3]
      if line.startswith('T'):
        t_seq+=re.split('\s+', line)[3]        
    ali=seq.AlignmentHandle()
    ali.AddSequence(seq.Sequence.FromString('query', q_seq))
    ali.AddSequence(seq.Sequence.FromString('target', t_seq))    
    return ali



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import info
from ost import gfx

class VisibilityOp:
  VISIBLE_ATTRIBUTE_NAME = "Visible"
  FLAGS_ATTRIBUTE_NAME = "Flags"
    
  def __init__(self, selection, flags, visible=False):    
    self.selection_ = selection
    self.visible_ = visible
    self.flags_ = flags
    
  def GetName(self):
    return "Visible: %s"%str(self.IsVisible())
  
  def SetSelection(self, selection):
    self.selection_ = selection
    
  def GetSelection(self):
    return self.selection_

  def SetVisible(self, visible):
    self.visible_ = visible

  def SetSelectionFlags(self, flags):
    self.flags_ = flags
    
  def GetSelectionFlags(self):
    return self.flags_

  def IsVisible(self):
    return self.visible_

  def ApplyOn(self, entity):
    if (entity is not None) and isinstance(entity, gfx.Entity):
      entity.SetVisible(entity.view.Select(self.GetSelection(),self.GetSelectionFlags()),self.IsVisible())
  
  def ToInfo(self,group):
      group.SetAttribute(VisibilityOp.VISIBLE_ATTRIBUTE_NAME, str(int(self.IsVisible())))
      group.SetAttribute(VisibilityOp.FLAGS_ATTRIBUTE_NAME, str(self.GetSelectionFlags()))
      group.SetTextData(str(self.GetSelection()))
    
  @staticmethod
  def FromInfo(group):
    visible_op = None
    if group.HasAttribute(VisibilityOp.VISIBLE_ATTRIBUTE_NAME):
      visible = bool(int(group.GetAttribute(VisibilityOp.VISIBLE_ATTRIBUTE_NAME)))
      flags = 0
      if group.HasAttribute(VisibilityOp.FLAGS_ATTRIBUTE_NAME):
        flags = int(group.GetAttribute(VisibilityOp.FLAGS_ATTRIBUTE_NAME))
      selection = group.GetTextData()
      visible_op = VisibilityOp(selection,flags,visible)
    return visible_op



import os, platform
import __main__
 
def GetPlatform():
  """
  Returns platform.system(). If the system call is interrupted, it is repeated.
  This function is a workaround for the buggy system call handling in Python.
  """
  system=None
  while not system:
    try:
      system=platform.system()
    except IOError:
      pass
  return system

def GetValue(val_key,val_default=None,prefix='OST'):
  """
  Returns the value of the variable val_key if defined, otherwise returns the 
  default value provided by the user (if provided). Search order: 
  
   * environment variable called $prefix_$val_key 
   * variable called val_key in .ostrc file
  """
  if prefix:
    env_var_name='%s_%s' % (prefix, val_key)
  else:
    env_var_name=val_key
  env_value=os.getenv(env_var_name)
  if env_value:
    return env_value
  else:
    main_attr=getattr(__main__, val_key, None)
    if main_attr:
      return main_attr
    else:
      return val_default

class FileNotFound(RuntimeError):
  """
  Raised when :func:`Locate` is unable to locate a file. The exception contains
  detailed information on what was tried to locate the file, i.e. search paths, 
  environment variables and also provides useful hints on how to let Locate know
  where to find the file.
  """
  def __init__(self, name, reason):
    self.name=name
    self.reason=reason
  def __str__(self):
    return 'Could not find "%s": %s' % (self.name, self.reason)

def Locate(file_name, explicit_file_name=None, search_paths=[],
           env_name=None, search_system_paths=True):
  """
  Helper function to locate files. To get the full name of an executable, let's 
  say qmake, use
  
  .. code-block:: python

    abs_qmake_path=Locate('qmake', env_name='QMAKE_EXECUTABLE')

  First the function checks if an environment variable with the name 
  QMAKE_EXECUTABLE is set. If so, the value of this variable is returned. Next, 
  each directory listed in search_paths is searched. If the executable could 
  still not be found and search_system_paths is set to True, the binary search 
  paths are searched.

  If the file could not be located, a :exc:`~ost.settings.FileNotFound` 
  exception will be raised containing a detail description why Locate failed. The 
  error message is formatted in such a way that it can directly be presented to 
  the user.
  """
  def _is_executable(filename):
    return os.path.exists(filename) and os.access(filename, os.X_OK)
  if type(file_name) is str:
    file_names=[file_name]
  else:
    file_names=file_name
  env_var_inexistent='env variable %s points to inexistent file %s'
  epxl_inexistent='explicitly set file "%s" does not exist'
  set_env_var='set the environment variable %s to the absolute path to %s or '
  if explicit_file_name:
    if _is_executable(explicit_file_name):
      return explicit_file_name
    else:
      raise FileNotFound(file_name, epxl_inexistent % explicit_file_name)
  if env_name:
    file_env_name=os.getenv(env_name, None)
    if file_env_name:
      if _is_executable(file_env_name):
        return file_env_name
      else:
        raise FileNotFound(file_name, 
                           env_var_inexistent % (env_name, file_env_name))
  searched=list(search_paths)
  for search_path in search_paths:
    for file_name in file_names:
      full_file_name=os.path.join(search_path, file_name)
      if _is_executable(full_file_name):
        return full_file_name

  if search_system_paths:
    paths=os.getenv('PATH')
    if GetPlatform() == "Windows":
      searched+=paths.split(';')
    else:
      searched+=paths.split(':')
    for path in searched:
      for file_name in file_names:
        full_file_name=os.path.join(path, file_name)
        if _is_executable(full_file_name):
          return full_file_name
  msg=''        
  if len(searched)>0:
    msg='searched in \n%s\n' % ( '\n'.join([' - %s' % s for s in searched]))
  if env_name:
    msg+=set_env_var % (env_name, ', ' % file_names)
  msg+='put %s into one of the search paths' % ', '.join(file_names)
  raise FileNotFound(file_name, msg)



import unittest
import ost
from ost import seq, io, mol

class TestAlignToSeqRes(unittest.TestCase):
  def testAlignWorking(self):
    ent, seqres = io.LoadMMCIF("testfiles/align_to_seqres.mmcif", seqres = True)
    chain =  ent.FindChain("A")
    sequence = seqres.FindSequence(chain.GetName());
    seqres_aln = seq.alg.AlignToSEQRES(chain, sequence, try_resnum_first=False)
    self.assertEqual(str(sequence), "MYTNSDFVVIKALEDGVNVIGLTRGADTRFHHSEKLDKGEV"+
                     "LIAQFTEHTSAIKVRGKAYIQTRHGVIESEGKK")
    self.assertEqual(str(seqres_aln.sequences[1]), "----SDFVVIKALEDGVNVIGLTR--"+
                     "-TRFHHSEKLDKGEVLIAQFTEHTSAIKVRGKAYIQTRHGVIESEGK-")

    seqres_aln = seq.alg.AlignToSEQRES(chain, sequence, try_resnum_first=True)
    self.assertEqual(str(seqres_aln.sequences[1]), "----SDFVVIKALEDGVNVIGLTR--"+
                     "-TRFHHSEKLDKGEVLIAQFTEHTSAIKVRGKAYIQTRHGVIESEGK-")

  def testAlignFail(self):
    ent, seqres = io.LoadMMCIF("testfiles/align_to_seqres_valueerror.mmcif",
                               seqres = True)
    chain =  ent.FindChain("A")
    sequence = seqres.FindSequence(chain.GetName());
    ost.PushVerbosityLevel(0)
    self.assertRaises(ValueError, seq.alg.AlignToSEQRES, chain, sequence, True)
    ost.PopVerbosityLevel()

  def testValidateWorking(self):
    ent, seqres = io.LoadMMCIF("testfiles/align_to_seqres.mmcif", seqres = True)
    chain =  ent.FindChain("A")
    sequence = seqres.FindSequence(chain.GetName());
    seqres_aln = seq.alg.AlignToSEQRES(chain, sequence, try_resnum_first=False)
    self.assertEqual(seq.alg.ValidateSEQRESAlignment(seqres_aln, chain), True)

  def testValidateWorkingOnAttachedView(self):
    ent, seqres = io.LoadMMCIF("testfiles/align_to_seqres.mmcif", seqres = True)
    chain =  ent.FindChain("A")
    sequence = seqres.FindSequence(chain.GetName());
    seqres_aln = seq.alg.AlignToSEQRES(chain, sequence, try_resnum_first=False)
    seqres_aln.AttachView(1, chain.Select(''))
    self.assertEqual(seq.alg.ValidateSEQRESAlignment(seqres_aln), True)

  def testValidateEmptySequenceWorking(self):
    alignment = seq.CreateAlignment(seq.CreateSequence('SEQRES', ''), 
                                    seq.CreateSequence('atoms', ''))
    chain = mol.ChainHandle()
    self.assertEqual(seq.alg.ValidateSEQRESAlignment(alignment, chain), True)

  def testValidateStrandBreakageFail(self):
    ent, seqres = io.LoadMMCIF("testfiles/validate_segres_aln_breakage.mmcif",
                               seqres = True)
    chain = ent.FindChain("A")
    sequence = seqres.FindSequence(chain.GetName());
    seqres_aln = seq.alg.AlignToSEQRES(chain, sequence, try_resnum_first = True,
                                       validate = False)
    self.assertEqual(seq.alg.ValidateSEQRESAlignment(seqres_aln, chain), False)

  def testValidateGapConnectedFail(self):
    ent, seqres = io.LoadMMCIF("testfiles/validate_seqres_aln_connected.mmcif",
                               seqres = True,)
    chain = ent.FindChain("A")
    sequence = seqres.FindSequence(chain.GetName());
    seqres_aln = seq.alg.AlignToSEQRES(chain, sequence, try_resnum_first = True,
                                       validate = False)
    self.assertEqual(seq.alg.ValidateSEQRESAlignment(seqres_aln, chain), False)


if __name__ == "__main__":
  from ost import testutils
  if testutils.SetDefaultCompoundLib():
    testutils.RunTests()
  else:
    print('No compound library available. Ignoring test_aligntoseqres.py tests.')



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from PyQt5 import QtCore, QtWidgets, QtGui

#Gradient Stop  
class ColorSelectWidget(QtWidgets.QWidget):

  colorChanged = QtCore.pyqtSignal()

  def __init__(self, width, height, color, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    
    #Membervars
    self.width_ = width
    self.height_ = height

    if(color is None):
      self.color_ = QtWidgets.QColor("White")
    else:
      self.color_ = color
        
    self.show()
    
    #ContextMenu
    self.change_color_ = QtWidgets.QAction('ChangeColor', self)
    
    self.change_color_.triggered.connect(self.ChangeColor)
    
    self.addAction(self.change_color_)
    self.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
        
    self.Resize()
    
  def ChangeColor(self):
    color = QtWidgets.QColorDialog.getColor(initial = self.color_, parent = self,
                                            title = "Select Color")
    
    if(color != self.color_ and color.isValid()):
      self.color_ = color
      self.colorChanged.emit()
      self.update()
    
  def GetColor(self):
    return self.color_
  
  def GetGfxColor(self):
    color = self.GetColor()
    return gfx.RGB(color.redF(), color.greenF(), color.blueF())
  
  def SetColor(self, color):
    if(self.color_ != color):
      self.color_ = color
      self.colorChanged.emit()
      self.update()
  
  def SetGfxColor(self, color):
    qcolor= QtWidgets.QColor(color.Red()*255,color.Green()*255,color.Blue()*255,color.Alpha()*255)
    self.SetColor(qcolor)
            
  def paintEvent(self, event):
    if self.isEnabled():
      size = self.size()
      paint = QtGui.QPainter()
      if paint.begin(self):
        brush = QtGui.QBrush(self.color_)
        paint.setBrush(brush)
        paint.drawRect(0,
                       0,
                       self.width() - 1,
                       self.height() - 1)
        paint.end()
      
  def mouseDoubleClickEvent(self, event):    
    self.ChangeColor()
   
  def SetSize(self, width, height):
    self.width_ = width
    self.height_ = height
    self.Resize()
    
  def Resize(self):
    self.setMinimumSize(self.width_, self.height_)
    self.setMaximumSize(self.width_, self.height_)  
    self.resize(self.width_, self.height_)



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

import sys
from ost import gui
from ost import gfx
from PyQt5 import QtCore, QtWidgets, QtGui

class ToolBarOptionsWidget(QtWidgets.QWidget):
  """QWidget with a ToolBar and a show area.
    
   This abstract QWidget has a toolbar and a show area. Whenever a button of the tool bar is pressed, the Widget corresponding to the button's value is shown in the show area.  
  """
  def __init__(self, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    
    #Setup ui_
    self.parent_=parent
    self.resize(400, 300)
    self.setMinimumSize(QtCore.QSize(250, 200))
    self.gridLayout = QtWidgets.QGridLayout(self)
    self.gridLayout.setHorizontalSpacing(0)
    self.gridLayout.setVerticalSpacing(0)
    self.gridLayout.setContentsMargins(0,0,0,0)
    self.gridLayout.setSpacing(0)    
    self.tool_bar_ = QtWidgets.QToolBar(self)
    self.tool_bar_.setIconSize(QtCore.QSize(16, 16))
    self.gridLayout.addWidget(self.tool_bar_, 0, 0, 1, 1)
    self.stackedWidget = QtWidgets.QStackedWidget(self)
    self.gridLayout.addWidget(self.stackedWidget, 1, 0, 1, 1)

    self.current_action_ = None
    self.actions_ = list()
    
    self.tool_bar_.actionTriggered.connect(self.ChangeSelectedItem)
    
    self.setEnabled(False)
           
    self.Update()
  
  def Update(self):
    """Updates the active widget of the show area.
    
     This method calls the Update method of the active widget.
    """
    self.setEnabled(True)
    widget = self.__GetCurrentWidget()
    if hasattr(widget, "Update"):
      widget.Update()
  
    
  def AddWidget(self, ident, widget, text=None):
    """Adds a Widget to this Options Widget.
    
     The Widget must have a identifier. If another Widget has the same identifier,
     the old widget will be removed and the new widget gets the identifier.
     Returns True, if widget is added. Otherwise it returns False
    """
    if isinstance(widget, QtWidgets.QWidget) and ident is not None:
      if text is not None:
        string = text
      elif hasattr(widget, "GetText"):
        string = widget.GetText()
      else:
        string = ident
      
      self.stackedWidget.addWidget(widget)
      action = self.tool_bar_.addAction(ident)
      action.setIcon(QtGui.QIcon(ident))
      action.setToolTip(string)
      pair = ident, widget
      action.setData(QtCore.QVariant(pair))
      action.setCheckable(True);
      if(len(self.actions_) == 0):
        self.ChangeSelectedItem(action)
      self.actions_.append(action)
      return True
    return False
  
  def OnComboChange(self, item):
    """This abstract method is called whenever the View is updated.
    
     This abstract method must be implemented by all subclasses. 
     It can be used to do something ;-) whenever the combobox changes its value.
    """
    raise NotImplementedError("Subclasses must define OnComboChange()")
  
  def DoResize(self):
    item = self.__GetCurrentWidget()
    width = 0
    height = 0
    if(hasattr(item,"minimumHeight")):
      height=item.minimumHeight()
    if(hasattr(item,"minimumWidth")):
      width=item.minimumWidth()
    self.setMinimumSize(width,self.tool_bar_.height()+height)
    if(hasattr(self.parent_,"DoResize")):
      self.parent_.DoResize()
  
  def ChangeSelectedItem(self, action):
    """Change Current Selected Item.
    
     Shows the widget which corresponds to the action in the show area.
    """
    if(self.current_action_ != None):
        self.current_action_.setChecked(False)
    else:
       self.current_action_ = action
    widget = action.data()[1]
    self.stackedWidget.setCurrentWidget(widget)
    if hasattr(widget, "Update"): 
      widget.Update()
    if(self.current_action_ == action):
      self.current_action_.setChecked(True)
    else:
      self.current_action_=action
    self.OnComboChange(widget)
  #Private Methods
  def __GetCurrentWidget(self):
      return self.stackedWidget.currentWidget();

  #Overwritten Methods
  def setEnabled(self, bool):
    QtWidgets.QWidget.setEnabled(self, bool)




import unittest
from ost import *
import subprocess

class TestPDB(unittest.TestCase):
  def setUp(self):
    pass

  def test_compnd_parser(self):
    e=io.LoadPDB('testfiles/pdb/compnd.pdb', restrict_chains="A")
    self.assertEqual(e.GetChainCount(), 1)
    ch = e.FindChain("A");
    self.assertEqual(ch.GetIntProp("mol_id"), 1)

  def test_properly_assigns_profile_properties(self):
    io.profiles['TEST'] = io.IOProfile()
    io.profiles['TEST'].quack_mode = False
    self.assertFalse(io.profiles['TEST'].quack_mode)
    self.assertFalse(io.profiles['TEST'].Copy().quack_mode)
    io.profiles['TEST'].quack_mode = True
    self.assertTrue(io.profiles['TEST'].quack_mode)
    self.assertTrue(io.profiles['TEST'].Copy().quack_mode)
  def test_no_bond_feasibility(self):
    io.profiles['FEAS_CHECK']=io.IOProfile(processor=conop.HeuristicProcessor(check_bond_feasibility=True))
    io.profiles['NO_FEAS_CHECK']=io.IOProfile(processor=conop.HeuristicProcessor(check_bond_feasibility=False))
        
    e1=io.LoadPDB('testfiles/pdb/simple_defective.pdb', restrict_chains="A",profile='FEAS_CHECK')
    e2=io.LoadPDB('testfiles/pdb/simple_defective.pdb', restrict_chains="A",profile='NO_FEAS_CHECK')
    ed=io.LoadPDB('testfiles/pdb/simple_defective.pdb', restrict_chains="A")
    
    res1=e1.FindResidue('A',3)
    res2=e2.FindResidue('A',3)
    resd=ed.FindResidue('A',3)

    self.assertFalse(mol.BondExists(res1.FindAtom("CA"),res1.FindAtom("CB")))
    self.assertTrue(mol.BondExists(res2.FindAtom("CA"),res2.FindAtom("CB")))
    self.assertTrue(mol.BondExists(resd.FindAtom("CA"),resd.FindAtom("CB")))

  def test_remote_loading(self):

    if subprocess.call(['ping google.com -c 1'], shell=True,
                       stdout=subprocess.PIPE, stderr=subprocess.PIPE) != 0:
      print("No internet connection (or wrong OS) to test remote loading in "
            "io.LoadPDB: ignoring unit test")
      return

    with self.assertRaises(IOError):
      io.LoadPDB('1ake', remote=True, remote_repo="cheeseisgoodforyou")

    # let's hope that crambin stays the same forever
    crambin_pdb = io.LoadPDB('1crn', remote=True, remote_repo='pdb')
    self.assertEqual(len(crambin_pdb.residues), 46)
    self.assertEqual(len(crambin_pdb.atoms), 327)
    
if __name__== '__main__':
  from ost import testutils
  testutils.RunTests()


 



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from ost import info
from PyQt5 import QtCore, QtWidgets

#Gradient Editor
class ImmutableInfoHandler:
  def __init__(self, fileName):
    
    self.FILE_NAME = fileName    
    self.GRADIENTS_GROUP_NAME = "Gradients"
    self.GRADIENT_GROUP_NAME = "Gradient"
    self.NAME_ATTRIBUTE_NAME = "Name"
    
    self.handle_ = info.LoadOrCreateInfo(self.FILE_NAME)
        
    self.root_ = self.handle_.Root()
    
    self.gradients_ = self.root_.RetrieveGroup(self.GRADIENTS_GROUP_NAME)

    
  def GetGfxGradient(self, name):
    group = self.gradients_.GetGroup(name)
    gfxGradient = gfx.Gradient.GradientFromInfo(group)
    return gfxGradient
  
  def GetQGradient(self, name):
    gfxGradient = self.GetGfxGradient(name)
    return self.ConvertToQGradient(gfxGradient)
  
  def ConvertToQGradient(self, gfxGradient):
    stops = gfxGradient.GetStops()
    qgrad = QtWidgets.QLinearGradient()
    for stop in stops:
      color = stop.GetColor()
      qcolor = QtWidgets.QColor(color.Red()*255,
                                color.Green()*255,
                                color.Blue()*255,
                                color.Alpha()*255)
      qgrad.setColorAt(stop.GetRel(), qcolor) 
    return qgrad 
  
  def ConvertToGfxGradient(self, gradient):  
    gfxgradient = gfx.Gradient()    
    for s in gradient.stops():
      rel=s[0]
      color=s[1]
      gfxgradient.SetColorAt(s[0], gfx.RGB(s[1].redF(), s[1].greenF(), s[1].blueF()));
    return gfxgradient;
  
  def GetQGradients(self):
    groupList = self.gradients_.GetGroups(self.GRADIENT_GROUP_NAME)
    qGradients = dict()
    
    for group in groupList:
      if group.HasAttribute(self.NAME_ATTRIBUTE_NAME):
        gfxGradient = gfx.Gradient.GradientFromInfo(group)
        qGradients[group.GetAttribute(self.NAME_ATTRIBUTE_NAME)] = (self.ConvertToQGradient(gfxGradient))
      
    return qGradients



import math
import sys
import ost.img.alg

# Tweakable parameters
wedge_start = 20 # Width of wedge at the narrower point
wedge_end = 500 # Width of wedge at the broader point
number_of_bands = 11 # Number of alternating bands (must be odd)
size_of_image_x = 500 # Size of the image (X)
size_of_image_y = 500 # Size of the image (Y)
pixel_sampling = 1.0 * Units.A # Pixel width
threshold = 2.0 * Units.A # Threshold for low pass filtering
amplitude = 255 # amplitude of the obscillations

# Image is created
image=img.CreateImage(img.Size(size_of_image_x,size_of_image_y))
image.CenterSpatialOrigin()
image.SetSpatialSampling(pixel_sampling)

# For a cleaner code, 4 shorthands are defined
image_extent=image.GetExtent()
start_y=image_extent.GetStart()[1]
end_y=image_extent.GetEnd()[1]
start_x=image_extent.GetStart()[0]
end_x=image_extent.GetEnd()[0]

# Wedge is written in the image
for y in range (start_y,end_y+1):
  wedge_width=wedge_start+(y-start_y)*(wedge_end-wedge_start)/(end_y-start_y)
  for x in range (start_x,end_x+1):
    half_wedge=wedge_width/2.0
    outer_bands=(number_of_bands-1)/2
    factor=(0.5+outer_bands)*math.pi/half_wedge
    if float(x)>-half_wedge and float(x)<half_wedge:
      value=255*math.cos(float(x)*factor)
      image.SetReal(img.Point(x,y),value)

# Image is low-pass filtered
filter=ost.img.alg.GaussianLowPassFilter(threshold)
image.ApplyIP(filter)

# Viewer is launched to show the result
v=gui.CreateDataViewer(image,"Modulated Image")



import unittest
from ost import *
from ost import settings
from ost import seq

class TestWeightMatrix(unittest.TestCase):
  
  def test_GetWeight(self):
    mat = seq.alg.BLOSUM62
    self.assertEqual(mat.GetWeight('A', 'A'), 4)
    self.assertEqual(mat.GetWeight('A', 'B'), -2)
    self.assertEqual(mat.GetWeight('a', 'A'), 4)
    self.assertEqual(mat.GetWeight('A', 'b'), -2)
    self.assertEqual(mat.GetWeight('A', '?'), 0)
    self.assertEqual(mat.GetWeight('E', '?'), 0)
    self.assertEqual(mat.GetWeight('Y', '?'), 0)
    self.assertEqual(mat.GetWeight('?', 'A'), 0)
    self.assertEqual(mat.GetWeight('?', 'E'), 0)
    self.assertEqual(mat.GetWeight('?', 'Y'), 0)
    self.assertEqual(mat.GetWeight('?', 'y'), 0)
    
    
  def test_SetWeight(self):
    mat = seq.alg.BLOSUM62
    self.assertEqual(mat.GetWeight('A', 'A'), 4)
    mat.SetWeight('A', 'A', -1)
    self.assertEqual(mat.GetWeight('A', 'A'), -1)
    
    self.assertEqual(mat.GetWeight('A', 'B'), -2)
    mat.SetWeight('A', 'B', 10)
    self.assertEqual(mat.GetWeight('A', 'B'), 10)
    
    self.assertEqual(mat.GetWeight('E', '?'), 0)
    mat.SetWeight('E', '?', 10)
    self.assertEqual(mat.GetWeight('E', '?'), 0)
    
    self.assertEqual(mat.GetWeight('?', 'E'), 0)
    mat.SetWeight('?', 'E', 10)
    self.assertEqual(mat.GetWeight('?', 'E'), 0)

if __name__ == "__main__":
  from ost import testutils
  testutils.RunTests()



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------

import unittest
if __name__== '__main__':
  import sys
  sys.path.insert(0,"../../../stage/lib64/openstructure/")
  sys.path.insert(0,"../../../stage/lib/openstructure/")

import ost
import ost.mol as mol
import ost.gfx as gfx
import ost.geom as geom

if ost.WITH_NUMPY:
  has_numpy=True
  try:
    import numpy
  except ImportError as e:
    has_numpy=False
else:
  has_numpy=False

def col_delta(c1,c2):
  return geom.Distance(geom.Vec3(c1[0],c1[1],c1[2]),geom.Vec3(c2[0],c2[1],c2[2]))

class MyGfxObj(gfx.GfxObj):
  def __init__(self,name):
    gfx.GfxObj.__init__(self,name)
    self.rendered=False
    
  def CustomRenderGL(self,render_pass):
    self.rendered=True

class TestGfx(unittest.TestCase):
  def runTest(self):
    self.test_gradient()
    self.test_color()
    self.test_primlist()
    self.test_entity_reset()
    self.test_custom_gfx_obj()
    self.test_gfxobj_conv()

  def test_gfxobj_conv(self):
    e=mol.CreateEntity()
    gfx.Scene().Add(gfx.Entity("foo2",e))
    gfx.Scene()["foo2"].SetColor(gfx.YELLOW)
    
  def test_custom_gfx_obj(self):
    myobj=MyGfxObj("foo")
    gfx.Scene().Add(myobj)
    #self.assertTrue(myobj.rendered)

  def test_entity_reset(self):
    eh=mol.CreateEntity()
    eh2=mol.CreateEntity()
    go=gfx.Entity("foo",eh)
    go.Reset(eh)
    go.Reset("rnum=1")
    go.Reset(entity=eh2)
    go.Reset(eh.Select("rnum=2"))
    go.Reset(eh,"rnum=3",mol.MATCH_RESIDUES)
    go.Reset("rnum=4",eh2)

  def test_gradient(self):
    gs=[gfx.Gradient(),
        gfx.Gradient({0.0: [1,0,0], 1.0: gfx.Color(0,1,0)}),
        gfx.Gradient([[1,0,0], gfx.Color(0,1,0)])]
    gs[0].SetColorAt(0.0,gfx.Color(1.0,0.0,0.0))
    gs[0].SetColorAt(1.0,gfx.Color(0.0,1.0,0.0))
    for g in gs:
      self.assertAlmostEqual(col_delta(g.GetColorAt(0.0),gfx.Color(1.0,0,0)),0.0)
      self.assertAlmostEqual(col_delta(g.GetColorAt(0.5),gfx.Color(0.5,0.5,0)),0.0)
      self.assertAlmostEqual(col_delta(g.GetColorAt(1.0),gfx.Color(0,1.0,0)),0.0)

  def test_color(self):
    c=gfx.Color(0.5,0.3,0.2)
    self.assertAlmostEqual(c.r,0.5)
    self.assertAlmostEqual(c.g,0.3)
    self.assertAlmostEqual(c.b,0.2)
    self.assertAlmostEqual(c.a,1.0)
    self.assertAlmostEqual(c.red,0.5)
    self.assertAlmostEqual(c.green,0.3)
    self.assertAlmostEqual(c.blue,0.2)
    self.assertAlmostEqual(c.alpha,1.0)
    self.assertAlmostEqual(c[0],0.5)
    self.assertAlmostEqual(c[1],0.3)
    self.assertAlmostEqual(c[2],0.2)
    self.assertAlmostEqual(c[3],1.0)
    c.r=0.9
    self.assertAlmostEqual(c.r,0.9)
    self.assertAlmostEqual(c.red,0.9)
    self.assertAlmostEqual(c[0],0.9)

  def test_primlist(self):
    pl=gfx.PrimList("foo")
    pl.AddPoint([0,0,0])
    pl.AddPoint(geom.Vec3(1,2,3),color=gfx.RED)
    pl.AddLine([0,0,0],[1,2,3])
    pl.AddLine(geom.Vec3(0,0,0),geom.Vec3(1,2,3),color=gfx.BLUE)
    pl.AddSphere([0,0,0],radius=2.0)
    pl.AddSphere(geom.Vec3(1,2,3),color=gfx.RED,radius=3.0)
    pl.AddCyl([0,0,0],[1,2,3],radius=0.5,color=gfx.YELLOW)
    pl.AddCyl(geom.Vec3(0,0,0),geom.Vec3(1,2,3),radius1=0.5,radius2=0.1,color1=gfx.BLUE,color2=gfx.GREEN)
    pl.AddText("foo",[0,2,3])
    pl.AddText("bar",[-2,0,0],color=gfx.WHITE,point_size=8)
    if has_numpy:
      pl.AddMesh(numpy.zeros((5,3),dtype=numpy.float32),
                 numpy.zeros((5,3),dtype=numpy.float32),
                 numpy.zeros((5,4),dtype=numpy.float32),
                 numpy.zeros((2,3),dtype=numpy.uint32))
      pl.AddMesh(numpy.zeros((7,3),dtype=numpy.float32),
                 None,
                 None,
                 numpy.zeros((4,3),dtype=numpy.uint32))

      # Passing wrong data type should fail
      with self.assertRaises(Exception):
        pl.AddMesh(numpy.zeros((5, 3), dtype=numpy.uint32),
                   numpy.zeros((5, 3), dtype=numpy.float32),
                   numpy.zeros((5, 4), dtype=numpy.float32),
                   numpy.zeros((2, 3), dtype=numpy.uint32))
      with self.assertRaises(Exception):
        pl.AddMesh(numpy.zeros((5, 3), dtype=numpy.float32),
                   numpy.zeros((5, 3), dtype=numpy.uint32),
                   numpy.zeros((5, 4), dtype=numpy.float32),
                   numpy.zeros((2, 3), dtype=numpy.uint32))
      with self.assertRaises(Exception):
        pl.AddMesh(numpy.zeros((5, 3), dtype=numpy.float32),
                   numpy.zeros((5, 3), dtype=numpy.float32),
                   numpy.zeros((5, 4), dtype=numpy.uint32),
                   numpy.zeros((2, 3), dtype=numpy.uint32))
      with self.assertRaises(Exception):
        pl.AddMesh(numpy.zeros((5, 3), dtype=numpy.float32),
                   numpy.zeros((5, 3), dtype=numpy.float32),
                   numpy.zeros((5, 4), dtype=numpy.float32),
                   numpy.zeros((2, 3), dtype=numpy.float32))
                 

if __name__== '__main__':
  from ost import testutils
  testutils.RunTests()




'''
MSMS module

Author: Tobias Schmidt

This module is for calculating MSMS surfaces as well as surface areas
(SESA, SASA) from OpenStructure using the external program MSMS.

How To Use This Module:
 1. Import it (e.g. as "from ost.bindings import msms")
 2. Use it (e.g. as "surfaces_list = msms.CalculateSurface(entity)"
                    "(sesa,sasa) = msms.CalculateSurfaceArea(entity)")

Requirement:
 - MSMS installed
'''

import tempfile
import subprocess
import os

from ost import io
from ost import mol
from ost import settings
from ost import geom


class MsmsProcessError(Exception):
  """
  Python 2.4 and older do not include the CalledProcessError exception. This
  class substitutes it.
  """
  def __init__(self, returncode,command):
    self.returncode = returncode
    self.command = command
  def __str__(self):
    return repr(self.returncode)


def GetVersion(msms_exe=None, msms_env=None):
  """
  Get version of MSMS executable
  """
  msms_executable = _GetExecutable(msms_exe, msms_env)
  command = "%s" % (msms_executable)
  proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
  stdout_value, stderr_value = proc.communicate()

  version = ""
  for l in stdout_value.decode().splitlines():
    if l[0:4]=='MSMS':
      version = l.split(' ')[1]
      return version
  if version=="":
    LogWarning('Could not parse MSMS version string')
    return


def _GetExecutable(msms_exe, msms_env):
  """
  Function to check if MSMS executable is present

  :param msms_exe: Explicit path to msms executable
  :param msms_env: Environment variable pointing to msms executable
  :returns: Path to the executable
  :raises:  :class:`~ost.FileNotFound` if executable is not found
  """
  return settings.Locate('msms', explicit_file_name=msms_exe,
                         env_name=msms_env)


def _SetupFiles(entity, selection):
  """
  Setup files for MSMS calculation in temporary directory

  :param entity: The entity for which the surface is to be calculated
  :type entity: :class:`~ost.mol.EntityHandle` or :class:`~ost.mol.EntityHandle`
  :param selection:  Calculate surface for subset of entity
  :type selection: :class:`str`
  :returns: tuple containing temporary directory and msms input file
  :raises:         :class:`RuntimeError` if selection is not valid
  """
  #   create temporary directory
  tmp_dir_name=tempfile.mkdtemp()

  # select only heavy atoms if no_hydrogens is true
  entity_view=entity.Select(selection)
  if not entity_view.IsValid():
    raise RuntimeError("Could not create view for selection (%s)"%(selection))

  # write entity to tmp file
  tmp_file_name=os.path.join(tmp_dir_name,"entity")
  tmp_file_handle=open(tmp_file_name, 'w')
  for a in entity_view.GetAtomList():
    position=a.GetPos()
    tmp_file_handle.write('%8.3f %8.3f %8.3f %4.2f\n' % (position[0],
                          position[1], position[2], a.radius))
  tmp_file_handle.close()
  
  return (tmp_dir_name, tmp_file_name)


def _ParseAreaFile(entity, selection, file, asa_prop, esa_prop):
  """
   Reads Area file (-af) and attach sasa and sesa per atom to an entitiy

  :param entity:   :class:`~ost.mol.EntityHandle` or :class:`~ost.mol.EntityView`
                   for attaching sasa and sesa on atom level
  :param file:     Filename of area file
  :param asa_prop: Name of the float property for SASA
  :param esa_prop: Name of the float property for SESA
  :raises: :class:`RuntimeError` if number of atoms in file != number of atoms in entity
  """
  view=entity.Select(selection)
  area_fh = open(file)
  area_lines = area_fh.readlines()
  area_fh.close()
  # shift first line
  area_lines = area_lines[1:]
  if view.GetAtomCount() != len(area_lines):
      raise RuntimeError("Atom count (%d) unequeal to number of atoms in area file (%d)" % (view.GetAtomCount(), len(area_lines)))
  for l in area_lines:
      atom_no, sesa, sasa = l.split()
      a = view.atoms[int(atom_no)]
      if asa_prop:
        a.SetFloatProp(asa_prop, float(sasa))
      if esa_prop:
        a.SetFloatProp(esa_prop, float(sesa))
    
    

def _CleanupFiles(dir_name):
  """
  Function which recursively deletes a directory and all the files contained
  in it. *Warning*: This method removes also non-empty directories without
  asking, so be careful!
  """
  import shutil
  shutil.rmtree(dir_name)

def _RunMSMS(command):
  """
  Run the MSMS surface calculation

  This functions starts the external MSMS executable and returns the stdout of
  MSMS.

  :param command:          Command to execute
  :returns:                 stdout of MSMS
  :raises:              :class:`CalledProcessError` for non-zero return value
  """
  proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
  stdout_value, stderr_value = proc.communicate()

  #check for successful completion of msms
  if proc.returncode!=0:
    print("WARNING: msms error\n", stdout_value.decode())
    raise MsmsProcessError(proc.returncode, command)

  return stdout_value.decode()
  


def CalculateSurfaceArea(entity, density=1.0, radius=1.5,  all_surf=False,
                         no_hydrogens=False, no_hetatoms=False, no_waters=False,
                         selection='',
                         msms_exe=None, msms_env=None, keep_files=False, 
                         attach_asa=None, attach_esa=None):
  """
  Calculates analytical solvent excluded and solvent accessible surface
  area by using the external MSMS program.

  This method calculates the molecular surface areas by invoking the external
  program MSMS. First, it is checked if the MSMS executable is present, then,
  the necessary files are prepared in a temporary directory and MSMS is
  executed. The last step is to remove the temporary directory.


  :param entity:        OST entity to calculate surface
  :param density:       Surface point density
  :param radius:       Surface probe radius
  :param all_surf:      Calculate surface area for all cavities (returns multiple
      surfaces areas as a list)
  :param no_hydrogens:  Calculate surface only for hevy atoms
  :param selection:     Calculate surface for subset of entity
  :param msms_exe:      msms executable (full path to executable)
  :param msms_env:      msms environment variable
  :param keep_files:    Do not delete temporary files
  :param attach_asa:    Attaches per atom SASA to specified FloatProp at atom level
  :param attach_esa:    Attaches per atom SESA to specified FloatProp at atom level
  :returns:             Tuple of lists for (SES, SAS)
  """
  import re 

  # check if msms executable is specified
  msms_executable=_GetExecutable(msms_exe, msms_env)

  # parse selection
  if no_hydrogens:
    if selection!='':
      selection+=" and "
    selection+="ele!=H"
  
  if no_hetatoms:
    if selection!='':
      selection+=" and "
    selection+="ishetatm=False"
  
  if no_waters:
    if selection!='':
      selection+=" and "
    selection+="rname!=HOH"

  # setup files for msms
  (msms_data_dir, msms_data_file)=_SetupFiles(entity, selection)

  # set command line
  command="%s -if %s -of %s -density %s -probe_radius %s -surface ases" % \
          (msms_executable, msms_data_file, msms_data_file, density, radius)
  if all_surf:
    command+=" -all"
  if attach_asa != None or attach_esa != None:
    command+=" -af %s" % os.path.join(msms_data_dir, "asa_atom")
  # run msms
  stdout_value=_RunMSMS(command)
  
  # add sesa and asa to entity if attach_asa is specified
  if attach_asa != None or attach_esa != None:
      _ParseAreaFile(entity, selection, os.path.join(msms_data_dir, "asa_atom.area"),
                     attach_asa, attach_esa)
  
  # parse MSMS output
  msms_ases=[]
  msms_asas=[]
  data_paragraph=False
  for line in stdout_value.splitlines():
    if re.match('MSMS terminated normally', line):
      data_paragraph=False
    if data_paragraph:
      (ses_,sas_)=line.split()[5:7]
      msms_ases.append(float(ses_))
      msms_asas.append(float(sas_))
    if re.match('    Comp. probe_radius,   reent,    toric,   contact    SES       SAS', line):
      data_paragraph=True

  # clean up
  if not keep_files:
    _CleanupFiles(msms_data_dir)

  return (msms_ases, msms_asas)
  
def CalculateSurfaceVolume(entity, density=1.0, radius=1.5,  all_surf=False,
                         no_hydrogens=False, no_hetatoms=False, no_waters=False,
                         selection='',
                         msms_exe=None, msms_env=None, keep_files=False, 
                         attach_asa=None, attach_esa=None):
  """
  Calculates the volume of the solvent excluded surface by using the external MSMS program.

  This method calculates the volume of the molecular surface by invoking the external
  program MSMS. First, it is checked if the MSMS executable is present, then,
  the necessary files are prepared in a temporary directory and MSMS is
  executed. The last step is to remove the temporary directory.


  :param entity:        OST entity to calculate surface
  :param density:       Surface point density
  :param radius:       Surface probe radius
  :param all_surf:      Calculate surface area for all cavities (returns multiple
      surfaces areas as a list)
  :param no_hydrogens:  Calculate surface only for hevy atoms
  :param selection:     Calculate surface for subset of entity
  :param msms_exe:      msms executable (full path to executable)
  :param msms_env:      msms environment variable
  :param keep_files:    Do not delete temporary files
  :param attach_asa:    Attaches per atom SASA to specified FloatProp at atom level
  :param attach_esa:    Attaches per atom SESA to specified FloatProp at atom level
  :returns:             Tuple of lists for (SES, SAS)
  """
  import re 

  # check if msms executable is specified
  msms_executable=_GetExecutable(msms_exe, msms_env)

  # parse selection
  if no_hydrogens:
    if selection!='':
      selection+=" and "
    selection+="ele!=H"
  
  if no_hetatoms:
    if selection!='':
      selection+=" and "
    selection+="ishetatm=False"
  
  if no_waters:
    if selection!='':
      selection+=" and "
    selection+="rname!=HOH"

  # setup files for msms
  (msms_data_dir, msms_data_file)=_SetupFiles(entity, selection)

  # set command line
  command="%s -if %s -of %s -density %s -probe_radius %s " % \
          (msms_executable, msms_data_file, msms_data_file, density, radius)
  if all_surf:
    command+=" -all"
  if attach_asa != None or attach_esa != None:
    command+=" -af %s" % os.path.join(msms_data_dir, "asa_atom")
  # run msms
  stdout_value=_RunMSMS(command)
  
  # add sesa and asa to entity if attach_asa is specified
  if attach_asa != None or attach_esa != None:
      _ParseAreaFile(entity, selection, os.path.join(msms_data_dir, "asa_atom.area"),
                     attach_asa, attach_esa)
  
  # parse MSMS output
  ses_volume=0
  for line in stdout_value.splitlines():
    if re.match('    Total ses_volume:', line):
      ses_volume=float(line.split(':')[1])

  # clean up
  if not keep_files:
    _CleanupFiles(msms_data_dir)

  return ses_volume


def CalculateSurface(entity, density=1.0, radius=1.5, all_surf=False,
                     no_hydrogens=False, no_hetatoms=False, no_waters=False,
                     selection='',
                     msms_exe=None, msms_env=None, keep_files=False,
                     attach_asa=None, attach_esa=None):
  
  """
  Calculates molecular surface by using the external MSMS program

  This method calculates a molecular surface by invoking the external program
  MSMS. First, it is checked if the MSMS executable is present, then, the
  necessary files are prepared in a temporary directory and MSMS is executed.
  The last step is to remove the temporary directory.


  :param entity:        Entity for which the surface is to be calculated
  :param density:       Surface point density
  :param radius:        Surface probe radius
  :param all_surf:      Calculate surface for all cavities (returns multiple
                        surfaces as a list)
  :param no_hydrogens:  Calculate surface only for heavy atoms
  :param selection:     Calculate surface for subset of entity
  :param msms_exe:      msms executable (full path to executable)
  :param msms_env:      msms environment variable
  :param keep_files:    Do not delete temporary files
  :param attach_asa:    Attaches per atom SASA to specified FloatProp at atom level
  :param attach_esa:    Attaches per atom SESA to specified FloatProp at atom level
  :returns:             list of :class:`~ost.mol.SurfaceHandle` objects
  """
  import os
  import re

  # check if msms executable is specified
  msms_executable=_GetExecutable(msms_exe, msms_env)

  # parse selection
  if no_hydrogens:
    if selection!='':
      selection+=" and "
    selection+="ele!=H"
      
  if no_hetatoms:
    if selection!='':
      selection+=" and "
    selection+="ishetatm=False"
      
  if no_waters:
    if selection!='':
      selection+=" and "
    selection+="rname!=HOH"

  # setup files for msms
  (msms_data_dir, msms_data_file)=_SetupFiles(entity, selection)

  # set command line
  command="%s -if %s -of %s -density %s -probe_radius %s" % (msms_executable,
          msms_data_file, msms_data_file, density, radius)
  if all_surf:
    command+=" -all"
  if attach_asa != None or attach_esa != None:
    command+=" -af %s" % os.path.join(msms_data_dir, "asa_atom")

  # run msms
  stdout_value=_RunMSMS(command)

  # add sesa and asa to entity if attach_asa is specified
  if attach_asa != None or attach_esa != None:
      _ParseAreaFile(entity, selection, os.path.join(msms_data_dir, "asa_atom.area"),
                     attach_asa, attach_esa)

  # parse msms output
  num_surf=0
  for line in stdout_value.splitlines():
    if re.search('RS component [0-9]+ identified',line):
      num_surf=int(line.split()[2])

  # get surfaces
  entity_sel = entity.Select(selection)
  msms_surfaces=[]
  s = io.LoadSurface(msms_data_file, "msms")
  s.Attach(entity_sel, 3+radius)
  msms_surfaces.append(s)
  for n in range(1,num_surf+1):
    filename=msms_data_file+'_'+str(n)
    s = io.LoadSurface(filename, "msms")
    s.Attach(entity_sel, 3+radius)
    msms_surfaces.append(s)

  # clean up
  if not keep_files:
    _CleanupFiles(msms_data_dir)

  return msms_surfaces




"""
deploy.py helps you package your code into a standalone application.

for now, macOS and Linux are supported...
"""
import os
import subprocess
import shutil
import sys
import glob

def _lib_name(component):
  return 'lib%s.dylib' % component

def _deps_for_lib(lib, pool, recursive=True):
  if lib in pool:
    return
  otool=subprocess.Popen(['otool', '-L', lib], stdout=subprocess.PIPE)
  output=otool.communicate()[0]
  lines=output.split('\n')[1:]
  for line in lines:
    d=line.split(' ')[0].strip()
    if len(d)>0:
      if d==lib:
        continue
      if d not in pool:
        if d.startswith('/System') or d.startswith('/usr/lib'):
          continue
        if recursive:
          _deps_for_lib(d, pool)
        pool.add(d)        
  return

def collect_deps(stage_dir, components, binaries, libexec_binaries,
                 site_packages, site_packages_dir, libexec_path='openstructure'):
  """
  Collect the dependencies for the given components and returns a list of 
  frameworks/libraries that the component depends on.
  """
  pool=set()
  for component in components:
    lib_name=os.path.abspath(os.path.join(stage_dir, 'lib', 
                                          _lib_name(component)))  
    if not os.path.exists(lib_name):
      print('WARNING:', lib_name, 'does not exist')
    if lib_name not in pool:
      _deps_for_lib(lib_name, pool)
      pool.add(lib_name)    
  for bin in binaries:  
    bin_name=os.path.abspath(os.path.join(stage_dir, 'bin', 
                                          bin))  
    if not os.path.exists(bin_name):
      print('WARNING:', bin_name, 'does not exist')
      continue
    if bin_name not in pool:
      _deps_for_lib(bin_name, pool)
  for bin in libexec_binaries:
    bin_name=os.path.abspath(os.path.join(stage_dir, 'libexec', libexec_path,
                                          bin))
    if not os.path.exists(bin_name):
      print('WARNING:', bin_name, 'does not exist')
      continue
    if bin_name not in pool:
      _deps_for_lib(bin_name, pool)
  for site_package in site_packages:
    full_path=get_python_module_path(site_package)
    print(full_path)
    if not os.path.exists(full_path):
      print('WARNING:', site_package, 'does not exists')
      continue
    if os.path.isdir(full_path):
      for so_file in glob.glob(os.path.join(full_path, '*.so')):
        _deps_for_lib(so_file, pool)
  return pool

LIBEXEC_SCRIPTS=['ost_config']
LIBEXEC_BINARIES=[]
GUI_LIBEXEC_BINARIES=['gosty']
BINARIES=['ldt', 'chemdict_tool', 'tmalign', 'tmscore']
GUI_BINARIES=[]
GUI_COMPONENTS=['gfx', 'gui', 'info']
COMPONENTS=['mol', 'geom', 'conop', 'seq_alg', 'seq',
            'img', 'img_alg', 'io', 'db', 'base']
GUI_SCRIPTS=['dng']
SCRIPTS=['ost']
CHANGE_ID_RPATH='install_name_tool -id @rpath/%s %s'   
CHANGE_ID='install_name_tool -id @rpath/%s %s'
CHANGE_LOAD_CMD_RPATH='install_name_tool -change %s @rpath/%s %s'
CHANGE_LOAD_CMD='install_name_tool -change %s @executable_path/%s %s'
ADD_RPATH='install_name_tool -add_rpath %s %s 2> /dev/null'
SITE_PACKAGES=[]
GUI_SITE_PACKAGES=['sip.so', 'sipconfig.py', 'sipdistutils.py', 'PyQt4']
REMOVE_HEADERS='rm -rf `find %s/lib -type d -name Headers`'
REMOVE_CURRENT='rm -rf `find %s/lib -type d -name Current`'
# collect libs of non-standard libraries/frameworks we depend on

def copy_binaries(stage_dir, outdir, binary_names, scripts, bin_dir,
                  append_bin=True):

  exe_path=os.path.abspath(os.path.join(outdir, bin_dir))
  for binary_name in binary_names:
    if append_bin:
      bin_name=os.path.join(stage_dir, bin_dir, binary_name)
    else:
      bin_name=os.path.join(stage_dir, binary_name)
    if not os.path.exists(bin_name):
      print('WARNING:', binary_name, 'does not exist')
      continue
    dst_name=os.path.join(outdir, bin_dir, os.path.basename(bin_name))
    shutil.copy(bin_name, dst_name)
    update_load_commands(dst_name, True, exe_path)
  for script in scripts:
    shutil.copy(os.path.join(stage_dir, bin_dir, script),
                os.path.join(outdir,bin_dir, script))

def split_framework_components(abs_path):
    """
    Splits the path pointing to a dynamic library within a framework
    
    '/System/Frameworks/A.framework/Versions/4/A' =>
    ['/System/Frameworks/A.framework', 'Versions/4/A']
    """
    parts=abs_path.split('/')
    for i, s in enumerate(parts):
      if s.endswith('.framework'):
        lead=os.path.join('/', *parts[:i+1])
        trail=os.path.join(*parts[i+1:])
        return lead, trail

def change_id(id, lib):
  os.chmod(lib, 0o666)
  os.system(CHANGE_ID_RPATH % (id,lib))
  os.chmod(lib, 0o444)

def update_load_commands(lib, exe, exe_path):
  direct_deps=set()
  _deps_for_lib(lib, direct_deps, recursive=False)
  os.chmod(lib, 0o666)
  for direct_dep in direct_deps:
    if direct_dep.endswith('.dylib'):
      new_name=os.path.basename(direct_dep)
      os.system(CHANGE_LOAD_CMD_RPATH % (direct_dep, new_name, lib))
    else:
      assert direct_dep.find('.framework/')>=0
      framework_path, rel_path=split_framework_components(direct_dep)
      framework_name=os.path.basename(framework_path)
      new_name=os.path.join(framework_name, rel_path)
      os.system(CHANGE_LOAD_CMD_RPATH % (direct_dep, new_name, lib))
  if exe:
    os.chmod(lib, 0o555)
  else:
    os.chmod(lib, 0o444)

def copy_deps(dependencies, outdir):
  exe_path=os.path.join(outdir, 'bin')
  for dep in dependencies:
    if dep.endswith('.dylib'):
      dst_name=os.path.join(outdir, 'lib', os.path.basename(dep))
      if not os.path.exists(dep):
        continue
      if os.path.exists(dst_name):
        continue
      shutil.copy(dep, dst_name)
      change_id(os.path.basename(dep), dst_name)
      update_load_commands(dst_name, False, exe_path)
    else:
      assert dep.find('.framework/')>=0
      framework_path, rel_path=split_framework_components(dep)
      framework_name=os.path.basename(framework_path)
      dst_name=os.path.join(outdir, 'lib', framework_name)
      shutil.copytree(framework_path, dst_name)
      change_id(os.path.join(dst_name, rel_path),
                os.path.join(dst_name, rel_path))
      os.unlink(os.path.join(dst_name, os.path.splitext(framework_name)[0]))
      update_load_commands(os.path.join(dst_name, rel_path), False, 
                           exe_path)

def update_pymod_shared_objects(lib_path, path, files):
  exe_path=os.path.abspath(os.path.join(lib_path, '../bin'))
  for f in files:
    if not os.path.exists(os.path.join(path, f)):
      continue
    base, ext=os.path.splitext(f)
    if  ext=='.so':
      path_to_lib_path=os.path.relpath(lib_path, path)
      abs_name=os.path.join(path, f)
      os.system(ADD_RPATH % (path_to_lib_path, abs_name))
      update_load_commands(abs_name, False, exe_path)
    elif ext in ('.pyc', '.pyo'):
      os.unlink(os.path.join(path, f))

def merge_tree(src, dst):
  """
  Similar to shutil.copytree, but does not complain when the destination
  directory already exists.
  """
  names = os.listdir(src)
  if not os.path.exists(dst):
    os.makedirs(dst)
  errors = []
  for name in names:
    srcname = os.path.join(src, name)
    dstname = os.path.join(dst, name)
    try:
        if os.path.islink(srcname):
            linkto = os.readlink(srcname)
            os.symlink(linkto, dstname)
        elif os.path.isdir(srcname):
            merge_tree(srcname, dstname)
        else:
            shutil.copy2(srcname, dstname)
    except (IOError, os.error) as why:
        errors.append((srcname, dstname, str(why)))
    except shutil.Error as err:
        errors.extend(err.args[0])
  try:
      shutil.copystat(src, dst)
  except OSError as why:
      if WindowsError is not None and isinstance(why, WindowsError):
          # Copying file access times may fail on Windows
          pass
      else:
          errors.extend((src, dst, str(why)))
  if errors:
      raise shutil.Error(errors)

def get_site_package_dir():
  """
  Get site-package directory of this python installation. This assumes 
  that ost was linked against the same version of Python (which is a very
  reasonable thing to do, as this script is most likely run with ost).
  """
  for p in sys.path:
    pattern='/site-packages'
    index=p.find(pattern)
    if index>=0:
      return p[:index+len(pattern)]
  raise RuntimeError("Couldn't determine site-packages location")

def get_python_module_path(module):
  for path in sys.path:
    full_path=os.path.join(path, module)
    if os.path.exists(full_path):
      return full_path
  return None
  
  
def get_python_home():
  """
  Derive Python home by looking at the location of the os module
  """
  return os.path.dirname(sys.modules['os'].__file__)


class Package(object):
  def __init__(self, name, root_dir, binaries=[], scripts=[], 
               modules=[], libraries=[], libexec_dir=None, libexec_scripts=[]):
    self.root_dir=root_dir
    self.name=name
    self.binaries=binaries
    self.scripts=scripts
    self.libraries=libraries
    self.libexec_dir=libexec_dir
    self.libexec_scripts=libexec_scripts
    self.pymod_dir=os.path.join('lib', 'python%d.%d' % sys.version_info[0:2],
                                'site-packages')
    self.modules=modules
    self.libexec_binaries=[]
    self.site_packages=[]
    self.site_packages_dir=''
  def status(self, message):
    print('%s: %s' % (self.name, message))

  def _prepare_output_dir(self, output_dir):
    """
    Prepares the output directory structure, including lib, bin and an optional
    libexec directory.
    """
    #if os.path.exists(output_dir):
    #  shutil.rmtree(output_dir)
    if not os.path.exists(output_dir):
      os.makedirs(output_dir)
    if not os.path.exists(os.path.join(output_dir, 'bin')):
      os.makedirs(os.path.join(output_dir, 'bin'))
    if not os.path.exists(os.path.join(output_dir, 'lib')):
      os.makedirs(os.path.join(output_dir, 'lib'))
    if self.libexec_dir:
      out_exec_dir=os.path.join(output_dir, 'libexec', self.libexec_dir)
      if not os.path.exists(out_exec_dir):
        print('making...', out_exec_dir)
        os.makedirs(out_exec_dir)
  def _copy_site_packages(self, output_dir):
    for sp in SITE_PACKAGES:
      src=get_python_module_path(sp)
      if os.path.isdir(src):
        merge_tree(src, os.path.joini(output_dir, self.pymod_dir, sp))
      else:
        shutil.copy(src, os.path.join(output_dir, self.pymod_dir, sp))
    print('updating link commands of python shared objects')
    os.path.walk(os.path.join(output_dir, 'lib'), 
                 update_pymod_shared_objects, 
                 os.path.join(output_dir, 'lib'))

  def ship(self, output_dir):
    self._prepare_output_dir(output_dir)
    if os.path.exists(os.path.join(self.root_dir, 'share')):
      self.status('copying shared data files')
      merge_tree(os.path.join(self.root_dir, 'share'), 
                 os.path.join(output_dir, 'share'))
    self.status('collecting dependencies')
    deps=collect_deps(self.root_dir, self.libraries, self.binaries, 
                      self.libexec_binaries, self.site_packages, 
                      self.site_packages_dir)
    # when running in non-gui mode, we are most likely missing the boost
    # python library. Let's add it to the list of dependencies by
    # inspecting "_ost_base.so".
    pymod_dir='lib/python%d.%d/site-packages' % sys.version_info[0:2]
    _deps_for_lib(os.path.join(self.root_dir, pymod_dir, 'ost/_ost_base.so'),
                  deps, recursive=False)
    self.status('copying dependencies')
    copy_deps(deps, output_dir)
    if self.libexec_dir:
      self.status('copying libexec binaries')
      copy_binaries(self.root_dir, output_dir, self.libexec_binaries, 
                    self.libexec_scripts,
                    os.path.join('libexec', self.libexec_dir))
    self.status('copying binaries')
    copy_binaries(self.root_dir, output_dir, self.binaries, 
                self.scripts, 'bin')
    self.status('copying pymod')
    merge_tree(os.path.join(self.root_dir,self.pymod_dir), 
               os.path.join(output_dir, self.pymod_dir))
    self._copy_site_packages(output_dir)

class OpenStructure(Package):
  def __init__(self, stage_dir, minimal=True):
    libs=['ost_mol', 'ost_geom', 'ost_conop', 'ost_seq_alg',
          'ost_io', 'ost_db', 'ost_base', 'ost_seq', 'ost_mol_alg']
    super(OpenStructure, self).__init__('OpenStructure', stage_dir, 
                                        binaries=['ldt', 'chemdict_tool'],
                                        libexec_scripts=['ost_config'],
                                        scripts=['ost'],
                                        libraries=libs,
                                        libexec_dir='openstructure')

class Qmean(Package):
  def __init__(self, stage_dir):
    super(Qmean, self).__init__('Qmean', stage_dir,
                                scripts=['qmean'],
                                libexec_scripts=['qmean_script.py'],
                                libexec_dir='qmean',
                                libraries=['qmean'])
ost=OpenStructure('../../stage')
qmean=Qmean('../../../../../qmean/stage')
ost.ship('qmean')
qmean.ship('qmean')




p=gfx.PrimList("p")
p.AddLine(geom.Vec3(0,0,0),geom.Vec3(5,0,0))
p.AddLine(geom.Vec3(0,0,0),geom.Vec3(0,5,0))
p.AddLine(geom.Vec3(0,0,0),geom.Vec3(0,0,5))
p.SetColor(gfx.YELLOW)
scene.Add(p)
p.SetLineWidth(5)
p.SetAALines(True)



# this script shows how to score a set of candidate fragments according to their 
# correlation with a density map. fragments with a higher correlation are ranked 
# better than fragments with a low correlation.
#

# tabula rasa first
scene.RemoveAll()

import functools
import ost.img.alg as img_alg
from ost.mol.alg import EntityToDensityRosetta, HIGH_RESOLUTION
sdh=io.LoadPDB('data/sdh-complete.pdb')
sdh_within=sdh.Select('rnum=94:101 and aname=CA,C,N,O')

padding=4
# we use a isotropic sampling of 1Anstroem
bounds=sdh_within.bounds
actual_size=bounds.max-bounds.min
size=img.Size(int(actual_size.x+2*padding), 
              int(actual_size.y+2*padding), 
              int(actual_size.x+2*padding))

def CreateDensityMap(size, origin):
  # create 3D image to hold the density information. Pixel sampling is 1A
  # by default. The image is empty
  dmap=img.CreateImage(size)
  dmap.SetAbsoluteOrigin(origin)  
  return dmap
origin=bounds.min-geom.Vec3(padding, padding, padding)
dmap=CreateDensityMap(size, origin)
cmap=CreateDensityMap(size, origin)

# create density for residue 94 to residue 101
EntityToDensityRosetta(sdh_within, dmap, HIGH_RESOLUTION, 5.0)
map_go=gfx.MapIso('Density', dmap, 0.0)
map_go.SetLevel(map_go.GetMean()*2.0)
scene.Add(map_go)
scene.CenterOn(map_go)

# load all loop candidates stored in the PDB file. note the use of the 
# load_multi parameter
candidates=io.LoadPDB('data/loop-candidates.pdb', 
                      load_multi=True)

print(len(candidates))
#-------------------------------------------------------------------------------
def Correlate(candidates):
  # this function converts the candidate loop into a density and calculates the 
  # real-spatial cross correlation with the actual density. The correlation 
  # coefficient is stored as the "correl" property
  
  for index, candidate in enumerate(candidates):
    EntityToDensityRosetta(candidate.CreateFullView(), cmap, 
                           HIGH_RESOLUTION, 5.0, True)
  
    correl=img_alg.RealSpatialCrossCorrelation(dmap, cmap, dmap.GetExtent())
    candidate.SetFloatProp('correl', correl)
#-------------------------------------------------------------------------------  
  
  
def Visualize(candidates):  
  # finally, sort the candidates by correlation and display them. 
  min_correl=min([c.GetFloatProp('correl') for c in candidates])
  max_correl=max([c.GetFloatProp('correl') for c in candidates])

  gradient=gfx.Gradient()
  gradient.SetColorAt(0.0, gfx.RED)
  gradient.SetColorAt(1.0, gfx.GREEN)

  # sort candidates by correlation
  def cmp_correl(lhs, rhs):
    lhs_correl=lhs.GetFloatProp('correl')
    rhs_correl=rhs.GetFloatProp('correl')
    if lhs_correl>rhs_correl:
      return -1
    elif lhs_correl==rhs_correl:
      return 0
    else:
      return 1
  candidates=sorted(candidates, key=functools.cmp_to_key(cmp_correl))

  # create render objects and add to scene
  for index, candidate in enumerate(candidates):
    candidate_go=gfx.Entity('candidate %03d' % index, candidate)
    norm=candidate.GetFloatProp('correl')/(max_correl-min_correl)
    candidate_go.SetColor(gradient.GetColorAt(norm))
    scene.Add(candidate_go)


Correlate(candidates)
Visualize(candidates)



import unittest
from ost import *
from ost import settings
from ost import testutils
from ost.seq.alg import SequenceIdentity
from ost.bindings import tmtools

class TestTMBindings(unittest.TestCase):
  
  def setUp(self):
    self.protein = io.LoadEntity("testfiles/testprotein.pdb")


  def testTMAlign(self):

    try:
      cad_calc_path = settings.Locate('tmalign')  
    except:
      print("Could not find tmalign executable: ignoring unit tests")
      return

    tm_result = tmtools.TMAlign(self.protein, self.protein)

    # model and reference are the same, we expect pretty good results
    self.assertEqual(tm_result.rmsd, 0.0)
    self.assertEqual(tm_result.tm_score, 1.0)
    self.assertEqual(tm_result.aligned_length, len(self.protein.chains[0].residues))
    self.assertEqual(SequenceIdentity(tm_result.alignment), 100.0)

    # transformation should be identity matrix (no transformation at all...)
    identity = geom.Mat4(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1)
    self.assertEqual(tm_result.transform, identity)


  def testTMScore(self):

    try:
      cad_calc_path = settings.Locate('tmscore')  
    except:
      print("Could not find tmalign executable: ignoring unit tests")
      return

    tm_result = tmtools.TMScore(self.protein, self.protein)

    # model and reference are the same, we expect pretty good results
    self.assertEqual(tm_result.rmsd_common, 0.0)
    self.assertEqual(tm_result.tm_score, 1.0)
    self.assertEqual(tm_result.max_sub, 1.0)
    self.assertEqual(tm_result.gdt_ts, 1.0)
    self.assertEqual(tm_result.gdt_ha, 1.0)
    self.assertEqual(tm_result.rmsd_below_five, 0.0)

    # transformation should be identity matrix (no transformation at all...)
    identity = geom.Mat4(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1)
    self.assertEqual(tm_result.transform, identity)


if __name__ == "__main__":
  testutils.RunTests()



import math,traceback

from PyQt5 import QtCore

import ost
from ost import gfx, gui

class SpacenavControl(QtCore.QObject):
  def __init__(self, spnav,
               parent=None):
    QtCore.QObject.__init__(self, parent)
    QtCore.QObject.connect(spnav,QtCore.SIGNAL("deviceTransformed(int,int,int,int,int,int)"), self.Changed)
    QtCore.QObject.connect(spnav,QtCore.SIGNAL("deviceButtonPressed(int)"), self.Toggle)     
    
    self.trans = True
    self.rot = True
    self.speed = 480.0
    
  def Changed(self, tx,ty,tz,rx,ry,rz):
    scene=gfx.Scene()
    tf = scene.transform
    def d(r):
      if r==0.0:
        return 0.0
      rr=r/abs(r)*max(0.0,abs(r)-0.9)
      if abs(rr)>0:
        return rr/abs(rr)*(math.pow(1.01,abs(rr))-1.0)*40.0/self.speed
      return 0.0
    if(self.trans):
      tf.ApplyXAxisTranslation(d(tx))
      tf.ApplyYAxisTranslation(d(ty))
      # adjust translation speed to distance from viewpoint
      currz=tf.trans[2];
      delta=currz*math.pow(1.01,d(tz))-currz;
      tf.ApplyZAxisTranslation(delta);
      # adjust near and far clipping planes together with z trans
      scene.SetNearFar(scene.near-delta,scene.far-delta);
    if(self.rot):
      tf.ApplyXAxisRotation(d(rx))
      tf.ApplyYAxisRotation(d(ry))
      tf.ApplyZAxisRotation(d(rz))

    scene.transform=tf
    scene.RequestRedraw()

  def Toggle(self, button):
    if button == 0:
      self.trans = not self.trans
      ost.LogVerbose("SpaceNav: translation %s" % (self.strans and "enabled" or "disabled"))
    elif button == 1:
      self.rot = not self.rot
      #ost.LogVerbose("SpaceNav: rotation %s"%("enabled" if self.rot else "disabled"))
      ost.LogVerbose("SpaceNav: rotation %s"%(self.rot and "enabled" or "disabled"))
    elif button == 12:
      if self.speed > 20:
        self.speed *= 0.8
        ost.LogVerbose("SpaceNav: speed increased to "+str(self.speed))
    elif button == 13:
      self.speed /= 0.8
      ost.LogVerbose("SpaceNav: speed reduced to "+str(self.speed))
    else:
      ost.LogVerbose("SpaceNav: unmapped button press ["+str(button)+"]")

      
def _InitSpaceNav(app):
  try:
    spnav = gui.SpnavInput.GetQThread()
    if spnav:
      spnav.start()
      parent = app.gl_win.qobject
      SpacenavControl(spnav,parent)
      ost.LogInfo("SpaceNav: device found and connected")
    else:
      ost.LogInfo("SpaceNav: no device found, or could not connect to device socket")
  except:
    ost.LogInfo("SpaceNav: caught exception during initialization: %s"%(traceback.format_exc()))



from ost import io,mol,geom
import unittest
import os
import random

class TestConvenientSuperpose(unittest.TestCase):
  
  def setUp(self):
    pass
  
  def assertEqualAtomOrder(self, view1, view2):
    self.assertEqual(len(view1.atoms),len(view2.atoms))
    for a1, a2 in zip(view1.atoms, view2.atoms):
      self.assertEqual(a1.element, a2.element)
      self.assertTrue(geom.Equal(a1.pos, a2.pos))

  def testAssertion(self):
    ent1_ent = io.LoadEntity(os.path.join("testfiles","1aho.pdb"))
    ent1_view = ent1_ent.Select("")
    self.assertEqualAtomOrder(ent1_ent, ent1_ent)
    self.assertEqualAtomOrder(ent1_ent, ent1_view)
    self.assertEqualAtomOrder(ent1_view, ent1_ent)
    self.assertEqualAtomOrder(ent1_view, ent1_view)
    self.assertRaises(AssertionError, self.assertEqualAtomOrder, ent1_view.Select("ele!=H"), ent1_view)
    
  def testCorrectlyOrdered(self):
    ent1_ent = io.LoadEntity(os.path.join("testfiles","1aho.pdb"))
    ent1_view = ent1_ent.Select("")
    ## test MatchResidueByNum
    view1, view2 = mol.alg.MatchResidueByNum(ent1_ent, ent1_ent)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByNum(ent1_view, ent1_ent)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByNum(ent1_ent, ent1_view)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByNum(ent1_view, ent1_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByIdx
    view1, view2 = mol.alg.MatchResidueByIdx(ent1_ent, ent1_ent)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByIdx(ent1_view, ent1_ent)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByIdx(ent1_ent, ent1_view)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByIdx(ent1_view, ent1_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByGlobalAln
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent1_ent, ent1_ent)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent1_view, ent1_ent)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent1_ent, ent1_view)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent1_view, ent1_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByLocalAln
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent1_ent, ent1_ent)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent1_view, ent1_ent)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent1_ent, ent1_view)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent1_view, ent1_view)
    self.assertEqualAtomOrder(view1, view2)

  def testMissingFirstAtom(self):
    ent_view = io.LoadEntity(os.path.join("testfiles","1aho.pdb")).Select("")
    ent_view_missing = ent_view.Select("not (cname=A and rnum=1 and aname=N)")
    ## test MatchResidueByNum
    view1, view2 = mol.alg.MatchResidueByNum(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByNum(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByIdx
    view1, view2 = mol.alg.MatchResidueByIdx(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByIdx(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByGlobalAln
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByLocalAln
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)

  def testMissingManyAtoms(self):
    ent_view = io.LoadEntity(os.path.join("testfiles","1aho.pdb")).Select("")
    ent_view_missing = ent_view.Select("not (cname=A and rnum=3,19,32 and aname=CB,CA,CD)")
    ## test MatchResidueByNum
    view1, view2 = mol.alg.MatchResidueByNum(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByNum(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByIdx
    view1, view2 = mol.alg.MatchResidueByIdx(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByIdx(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByGlobalAln
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByLocalAln
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)

  def testMissingFirstResidue(self):
    ent_view = io.LoadEntity(os.path.join("testfiles","1aho.pdb")).Select("")
    ent_view_missing = ent_view.Select("not (cname=A and rnum=1)")
    ## test MatchResidueByNum
    view1, view2 = mol.alg.MatchResidueByNum(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByNum(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByGlobalAln
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByLocalAln
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)

  def testMissingHydrogens(self):
    ent_view = io.LoadEntity(os.path.join("testfiles","1aho.pdb")).Select("")
    ent_view_missing = ent_view.Select("ele!=H")
    ## test MatchResidueByNum
    view1, view2 = mol.alg.MatchResidueByNum(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByNum(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByIdx
    view1, view2 = mol.alg.MatchResidueByIdx(ent_view, ent_view_missing)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByIdx(ent_view_missing, ent_view)
    self.assertEqualAtomOrder(view1, view2)
  
  def testWrongAtomOrder(self):
    ent_view = io.LoadEntity(os.path.join("testfiles","1aho.pdb")).Select("")
    ent_view_wrong = ent_view.CreateEmptyView()
    for c in ent_view.chains:
      ent_view_wrong.AddChain(c)
      for r in c.residues:
        ent_view_wrong.AddResidue(r)
        atoms = list(r.atoms)
        random.shuffle(atoms)
        for a in atoms:
          ent_view_wrong.AddAtom(a)
    ## test MatchResidueByNum
    view1, view2 = mol.alg.MatchResidueByNum(ent_view, ent_view_wrong)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByNum(ent_view_wrong, ent_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByIdx
    view1, view2 = mol.alg.MatchResidueByIdx(ent_view, ent_view_wrong)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByIdx(ent_view_wrong, ent_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByGlobalAln
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent_view, ent_view_wrong)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent_view_wrong, ent_view)
    self.assertEqualAtomOrder(view1, view2)
    ## test MatchResidueByLocalAln
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent_view, ent_view_wrong)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent_view_wrong, ent_view)
    self.assertEqualAtomOrder(view1, view2)

  def testWrongResidueOrder(self):
    ent_view = io.LoadEntity(os.path.join("testfiles","1aho.pdb")).Select("")
    ent_view_wrong = ent_view.CreateEmptyView()
    for c in ent_view.chains:
      ent_view_wrong.AddChain(c)
      residues = list(c.residues)
      random.shuffle(residues)
      for r in residues:
        ent_view_wrong.AddResidue(r)
        for a in r.atoms:
          av=ent_view_wrong.AddAtom(a)
    ## test MatchResidueByNum
    view1, view2 = mol.alg.MatchResidueByNum(ent_view, ent_view_wrong)
    self.assertEqualAtomOrder(view1, view2)
    view1, view2 = mol.alg.MatchResidueByNum(ent_view_wrong, ent_view)
    self.assertEqualAtomOrder(view1, view2)

  def testPeptideFilter(self):
    # check that CA only chains are ok and ligand chains are skipped
    ent_1_full = io.LoadPDB(os.path.join("testfiles", "5tglA.pdb"))
    ent_2_full = io.LoadPDB(os.path.join("testfiles", "5tglA_modified.pdb"))
    exp_atom_count = 253
    # check CA-only chain
    ent_1 = ent_1_full.Select("cname=A")
    ent_2 = ent_2_full.Select("cname=A")
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent_1, ent_2)
    self.assertEqual(view1.atom_count, exp_atom_count)
    self.assertEqual(view2.atom_count, exp_atom_count)
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent_1, ent_2)
    self.assertEqual(view1.atom_count, exp_atom_count)
    self.assertEqual(view2.atom_count, exp_atom_count)
    # check ligand chain
    ent_1_ligand = ent_1_full.Select("cname='_'")
    ent_2_ligand = ent_2_full.Select("cname='_'")
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent_1_ligand, ent_2_ligand)
    self.assertEqual(view1.atom_count, 0)
    self.assertEqual(view2.atom_count, 0)
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent_1_ligand, ent_2_ligand)
    self.assertEqual(view1.atom_count, 0)
    self.assertEqual(view2.atom_count, 0)
    # check both together
    view1, view2 = mol.alg.MatchResidueByLocalAln(ent_1_full, ent_2_full)
    self.assertEqual(view1.atom_count, exp_atom_count)
    self.assertEqual(view2.atom_count, exp_atom_count)
    view1, view2 = mol.alg.MatchResidueByGlobalAln(ent_1_full, ent_2_full)
    self.assertEqual(view1.atom_count, exp_atom_count)
    self.assertEqual(view2.atom_count, exp_atom_count)
    # try case where local alignment fails
    ev1 = ent_1.Select('rindex<2')   # seq = GI
    ev2 = ent_2.Select('rindex=2:3') # seq = RA
    view1, view2 = mol.alg.MatchResidueByLocalAln(ev1, ev2)
    self.assertEqual(view1.atom_count, 0)
    self.assertEqual(view2.atom_count, 0)

if __name__ == "__main__":
  from ost import testutils
  testutils.RunTests()



'''Documentation build on sphinx - please run from root dir by calling ost doc/make.py'''

import os, sys, re
import shutil
from ost import settings
from optparse import OptionParser
import subprocess
from sphinx.application import Sphinx

if len(sys.argv)==2:
  root_dir=sys.argv[1]
else:
  root_dir='.'

def _OutputPath(inpath, outdir):
  parts=inpath.split(os.path.sep)
  filtered_parts=[outdir]
  index=parts.index('modules')
  for part in parts[index+1:]:
    if part!='doc':
      filtered_parts.append(part)
  return os.path.join(*filtered_parts)

def _RequireCopy(in_name, out_name):
  if not os.path.exists(out_name):
    return True
  if os.path.getmtime(in_name)>os.path.getmtime(out_name):
    return True
  return False


pattern = re.compile(r'\.\.\s+image\:\:\s+([a-zA-Z0-9_\-//]+\.png|[a-zA-Z0-9_\-//]+\.jpg)')
def _CheckImage(in_name):
  file = open(in_name, "r", encoding='utf8')
  print("IN", in_name)
  text = file.read()
  picture_list = pattern.findall(text)
  file.close()
  return picture_list

def _CreateAndCopy(in_name, outdir):
  out_name=_OutputPath(in_name, outdir)
  out_dir=os.path.dirname(out_name)
  if not os.path.exists(out_dir):
    _MkDir(out_dir)
  if _RequireCopy(in_name, out_name):
    print('cp %s %s' % (in_name, out_name))
    os.system('cp %s %s' % (in_name, out_name))

def _MkDir(dirname):
  """
  Recursively create directories if they don't exist
  """
  parts=dirname.split(os.path.sep)
  for i, d in enumerate(parts):
    n=os.path.join(*parts[:1+i])
    if not os.path.exists(n):
      os.mkdir(n)

def _CollectRstDocs(outdir, dirname, fnames):
  for fname in fnames:
    if fname.endswith('.rst'):
      in_name=os.path.join(dirname, fname)
      _CreateAndCopy(in_name,outdir)
      img_list = _CheckImage(in_name)
      for img in img_list:
        img_name = os.path.join(dirname,img)
        _CreateAndCopy(img_name, outdir)

def ParseArgs():
  parser = OptionParser("usage: ost make.py [options] ")
  parser.add_option("-l","--linkcheck", action="store_true", default=False, dest="linkcheck", help="validate all external links")
  parser.add_option("-b", "--build-html", action="store_true",
                    default=False, dest="html", help="build html documentation")
  parser.add_option('-j', '--build-json', action='store_true', default=False)
  parser.add_option("-d", "--doctest", action="store_true", default=False, dest="doctest", help="run all test")
  parser.add_option("-q", "--quiet", action="store_true", default=False, dest="quiet", help="run all test")
  return parser.parse_args()

def _SelectBuilders(opts):
  builders = []
  if opts.html:
    builders.append('html')

  if opts.doctest:
    builders.append('doctest')

  if opts.build_json:
    builders.append('json')

  if opts.linkcheck:
    builders.append('linkcheck')
  return builders


opts, args=ParseArgs()
if not opts.html and\
   not opts.linkcheck and\
   not opts.doctest:
     opts.html=True

extra_args = {}
if opts.quiet:
  extra_args['warning'] = None
  extra_args['status'] = None

for sub_dir in ('modules',):
  for directory, dirnames, filenames in os.walk(sub_dir):
    _CollectRstDocs('doc/source', directory, filenames)

builders = _SelectBuilders(opts)

for builder in builders:
  Sphinx(
    srcdir='doc/source',
    confdir='doc/conf',
    outdir='doc/build/%s' % builder,
    doctreedir='doc/build/html/.doctrees',
    buildername=builder,
    **extra_args).build(True)
